"""
命名空间管理API - 重构版本
使用数据库持久化命名空间数据
"""
from fastapi import APIRouter, HTTPException, Query
from typing import Dict, Any, Optional, List
from datetime import datetime
import json
from sqlalchemy import text
import logging
from jettask.schemas import NamespaceCreate, NamespaceUpdate, NamespaceResponse

logger = logging.getLogger(__name__)

router = APIRouter(prefix="/api/namespaces", tags=["namespaces"])

# 使用任务中心专用的元数据库连接
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker
from jettask.backend.config import task_center_config

# 创建异步引擎 - 连接到任务中心元数据库
# 注意：这是任务中心自己的数据库，不是JetTask应用的数据库
engine = create_async_engine(task_center_config.meta_database_url, echo=False)
AsyncSessionLocal = sessionmaker(engine, class_=AsyncSession, expire_on_commit=False)


# 模型已从 schemas 模块导入


@router.get("", response_model=List[NamespaceResponse])
async def list_namespaces(
    page: int = Query(1, ge=1),
    page_size: int = Query(20, ge=1, le=100),
    is_active: Optional[bool] = None
):
    """列出所有命名空间"""
    try:
        async with AsyncSessionLocal() as session:
            query = """
                SELECT id, name, description, redis_config, pg_config, 
                       is_active, version, created_at, updated_at
                FROM namespaces
            """
            params = {}
            
            if is_active is not None:
                query += " WHERE is_active = :is_active"
                params['is_active'] = is_active
            
            query += " ORDER BY created_at DESC"
            query += " LIMIT :limit OFFSET :offset"
            params['limit'] = page_size
            params['offset'] = (page - 1) * page_size
            
            result = await session.execute(text(query), params)
            rows = result.fetchall()
            
            namespaces = []
            for row in rows:
                namespaces.append(NamespaceResponse(
                    id=row.id,
                    name=row.name,
                    description=row.description,
                    redis_config=row.redis_config,
                    pg_config=row.pg_config,
                    is_active=row.is_active,
                    version=row.version,
                    created_at=row.created_at,
                    updated_at=row.updated_at,
                    connection_url=f"/api/namespaces/{row.name}"  # 使用名称
                ))
            
            return namespaces
    except Exception as e:
        logger.error(f"Failed to list namespaces: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.post("", response_model=NamespaceResponse, status_code=201)
async def create_namespace(namespace: NamespaceCreate):
    """创建新的命名空间"""
    try:
        async with AsyncSessionLocal() as session:
            # 检查名称是否已存在
            check_query = text("SELECT COUNT(*) FROM namespaces WHERE name = :name")
            result = await session.execute(check_query, {'name': namespace.name})
            if result.scalar() > 0:
                raise HTTPException(status_code=400, detail=f"命名空间 '{namespace.name}' 已存在")
            
            # 创建命名空间（使用自增ID）
            insert_query = text("""
                INSERT INTO namespaces (name, description, redis_config, pg_config, version)
                VALUES (:name, :description, :redis_config, :pg_config, 1)
                RETURNING id, name, description, redis_config, pg_config, 
                          is_active, version, created_at, updated_at
            """)
            
            result = await session.execute(insert_query, {
                'name': namespace.name,
                'description': namespace.description,
                'redis_config': json.dumps(namespace.redis_config),
                'pg_config': json.dumps(namespace.pg_config)
            })
            
            row = result.fetchone()
            await session.commit()
            
            return NamespaceResponse(
                id=str(row.id),
                name=row.name,
                description=row.description,
                redis_config=row.redis_config,
                pg_config=row.pg_config,
                is_active=row.is_active,
                version=row.version,
                created_at=row.created_at,
                updated_at=row.updated_at,
                connection_url=f"/api/namespaces/{str(row.id)}"
            )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to create namespace: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.get("/{namespace_id}", response_model=NamespaceResponse)
async def get_namespace(namespace_id: str):
    """获取指定命名空间的详细信息"""
    try:
        async with AsyncSessionLocal() as session:
            query = text("""
                SELECT id, name, description, redis_config, pg_config, 
                       is_active, version, created_at, updated_at
                FROM namespaces
                WHERE id = :id
            """)
            
            result = await session.execute(query, {'id': namespace_id})
            row = result.fetchone()
            
            if not row:
                raise HTTPException(status_code=404, detail="命名空间不存在")
            
            return NamespaceResponse(
                id=str(row.id),
                name=row.name,
                description=row.description,
                redis_config=row.redis_config,
                pg_config=row.pg_config,
                is_active=row.is_active,
                version=row.version,
                created_at=row.created_at,
                updated_at=row.updated_at,
                connection_url=f"/api/namespaces/{str(row.id)}"
            )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to get namespace: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.put("/{namespace_id}", response_model=NamespaceResponse)
async def update_namespace(namespace_id: str, namespace: NamespaceUpdate):
    """更新命名空间"""
    try:
        async with AsyncSessionLocal() as session:
            # 检查是否存在并获取名称
            check_query = text("SELECT name FROM namespaces WHERE id = :id")
            result = await session.execute(check_query, {'id': namespace_id})
            row = result.fetchone()
            
            if not row:
                raise HTTPException(status_code=404, detail="命名空间不存在")
            
            # 检查是否是默认命名空间
            if row.name == 'default':
                raise HTTPException(status_code=400, detail="默认命名空间不能编辑")
            
            # 构建更新语句
            updates = []
            params = {'id': namespace_id}
            
            if namespace.description is not None:
                updates.append("description = :description")
                params['description'] = namespace.description
            
            if namespace.redis_config is not None:
                updates.append("redis_config = :redis_config")
                params['redis_config'] = json.dumps(namespace.redis_config)
            
            if namespace.pg_config is not None:
                updates.append("pg_config = :pg_config")
                params['pg_config'] = json.dumps(namespace.pg_config)
            
            if namespace.is_active is not None:
                updates.append("is_active = :is_active")
                params['is_active'] = namespace.is_active
            
            if not updates:
                raise HTTPException(status_code=400, detail="没有要更新的字段")
            
            # 如果更新了redis_config或pg_config，递增版本号
            if 'redis_config' in params or 'pg_config' in params:
                updates.append("version = version + 1")
            
            update_query = text(f"""
                UPDATE namespaces
                SET {', '.join(updates)}
                WHERE id = :id
                RETURNING id, name, description, redis_config, pg_config, 
                          is_active, version, created_at, updated_at
            """)
            
            result = await session.execute(update_query, params)
            row = result.fetchone()
            await session.commit()
            
            return NamespaceResponse(
                id=str(row.id),
                name=row.name,
                description=row.description,
                redis_config=row.redis_config,
                pg_config=row.pg_config,
                is_active=row.is_active,
                version=row.version,
                created_at=row.created_at,
                updated_at=row.updated_at,
                connection_url=f"/api/namespaces/{str(row.id)}"
            )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to update namespace: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.delete("/{namespace_id}")
async def delete_namespace(namespace_id: str):
    """删除命名空间"""
    try:
        async with AsyncSessionLocal() as session:
            # 检查是否为默认命名空间
            check_query = text("SELECT name FROM namespaces WHERE id = :id")
            result = await session.execute(check_query, {'id': namespace_id})
            row = result.fetchone()
            
            if not row:
                raise HTTPException(status_code=404, detail="命名空间不存在")
            
            if row.name == 'default':
                raise HTTPException(status_code=400, detail="不能删除默认命名空间")
            
            # 删除命名空间
            delete_query = text("DELETE FROM namespaces WHERE id = :id")
            await session.execute(delete_query, {'id': namespace_id})
            await session.commit()
            
            return {"message": "命名空间已删除"}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to delete namespace: {e}")
        raise HTTPException(status_code=500, detail=str(e))




"""
定时任务调度模块
支持 Redis + PostgreSQL 双存储方案
"""

from .models import ScheduledTask, TaskExecutionHistory
from .scheduler import TaskScheduler
from .loader import TaskLoader
from .manager import ScheduledTaskManager

__all__ = [
    'ScheduledTask',
    'TaskExecutionHistory', 
    'TaskScheduler',
    'TaskLoader',
    'ScheduledTaskManager'
]


"""
告警管理路由
提供轻量级的路由入口，业务逻辑在 AlertService 中实现
"""
from fastapi import APIRouter, HTTPException, Query
from typing import Optional
import logging

from jettask.schemas import AlertRuleRequest
from jettask.services.alert_service import AlertService

router = APIRouter(prefix="/alerts", tags=["alerts"])
logger = logging.getLogger(__name__)


@router.get("/rules")
async def get_alert_rules(
    page: int = Query(1, ge=1),
    page_size: int = Query(20, ge=1, le=100),
    is_active: Optional[bool] = None
):
    """
    获取告警规则列表
    
    Args:
        page: 页码（从1开始）
        page_size: 每页数量
        is_active: 是否只返回激活的规则
    """
    try:
        return AlertService.get_alert_rules(page, page_size, is_active)
    except Exception as e:
        logger.error(f"获取告警规则列表失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/rules", status_code=201)
async def create_alert_rule(request: AlertRuleRequest):
    """
    创建告警规则
    
    Args:
        request: 告警规则请求数据
    """
    try:
        return AlertService.create_alert_rule(request)
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        logger.error(f"创建告警规则失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.put("/rules/{rule_id}")
async def update_alert_rule(rule_id: str, request: AlertRuleRequest):
    """
    更新告警规则
    
    Args:
        rule_id: 规则ID
        request: 告警规则请求数据
    """
    try:
        return AlertService.update_alert_rule(rule_id, request)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"更新告警规则失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.delete("/rules/{rule_id}")
async def delete_alert_rule(rule_id: str):
    """
    删除告警规则
    
    Args:
        rule_id: 规则ID
    """
    try:
        return AlertService.delete_alert_rule(rule_id)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"删除告警规则失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/rules/{rule_id}/toggle")
async def toggle_alert_rule(rule_id: str):
    """
    启用/禁用告警规则
    
    Args:
        rule_id: 规则ID
    """
    try:
        return AlertService.toggle_alert_rule(rule_id)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"切换告警规则状态失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.get("/rules/{rule_id}/history")
async def get_alert_history(
    rule_id: str,
    page: int = Query(1, ge=1),
    page_size: int = Query(20, ge=1, le=100)
):
    """
    获取告警触发历史
    
    Args:
        rule_id: 规则ID
        page: 页码（从1开始）
        page_size: 每页数量
    """
    try:
        return AlertService.get_alert_history(rule_id, page, page_size)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"获取告警历史失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/rules/{rule_id}/test")
async def test_alert_rule(rule_id: str):
    """
    测试告警规则（发送测试通知）
    
    Args:
        rule_id: 规则ID
    """
    try:
        return AlertService.test_alert_rule(rule_id)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"测试告警规则失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


# ============ 告警统计接口 ============

@router.get("/statistics")
async def get_alert_statistics(
    namespace: Optional[str] = Query(None, description="命名空间名称")
):
    """
    获取告警统计信息
    
    Args:
        namespace: 命名空间名称（可选）
    """
    try:
        return AlertService.get_alert_statistics(namespace)
    except Exception as e:
        logger.error(f"获取告警统计信息失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


# ============ 活跃告警管理 ============

@router.get("/active")
async def get_active_alerts(
    namespace: Optional[str] = Query(None, description="命名空间名称")
):
    """
    获取当前活跃的告警
    
    Args:
        namespace: 命名空间名称（可选）
    """
    try:
        return AlertService.get_active_alerts(namespace)
    except Exception as e:
        logger.error(f"获取活跃告警失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/active/{alert_id}/acknowledge")
async def acknowledge_alert(
    alert_id: str,
    user: str = Query("system", description="确认用户")
):
    """
    确认告警
    
    Args:
        alert_id: 告警ID
        user: 确认用户
    """
    try:
        return AlertService.acknowledge_alert(alert_id, user)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"确认告警失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/active/{alert_id}/resolve")
async def resolve_alert(
    alert_id: str,
    resolution_note: Optional[str] = Query(None, description="解决说明")
):
    """
    解决告警
    
    Args:
        alert_id: 告警ID
        resolution_note: 解决说明（可选）
    """
    try:
        return AlertService.resolve_alert(alert_id, resolution_note)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e))
    except Exception as e:
        logger.error(f"解决告警失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


__all__ = ['router']


"""
Monitoring and analytics API v1
"""
from typing import List, Optional
from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.ext.asyncio import AsyncSession
import redis.asyncio as redis

from dependencies import (
    get_validated_namespace, get_pg_connection, get_redis_client,
    get_namespace_connection, validate_time_range, get_request_metrics, RequestMetrics
)
from models.requests import MonitoringRequest, BacklogTrendRequest, AnalyticsRequest
from models.responses import MonitoringResponse, AnalyticsResponse, BaseResponse
from core.cache import cache_result, CACHE_CONFIGS
from queue_backlog_api import get_backlog_trend
import logging

logger = logging.getLogger(__name__)
router = APIRouter()


@router.post("/backlog-trends", response_model=MonitoringResponse)
@cache_result(**CACHE_CONFIGS['monitoring_data'])
async def get_queue_backlog_trends(
    request: BacklogTrendRequest,
    namespace: str = Depends(get_validated_namespace),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取队列积压趋势（统一接口）"""
    metrics.start(namespace, "POST /monitoring/backlog-trends")
    
    try:
        # 直接调用现有的积压趋势API
        backlog_response = await get_backlog_trend(request)
        
        # 转换为标准监控响应格式
        monitoring_data = {
            'series': [],
            'granularity': backlog_response.granularity,
            'time_range': backlog_response.time_range
        }
        
        # 按系列分组数据
        series_data = {}
        for item in backlog_response.data:
            series_name = item.get('group') or item['queue']
            if series_name not in series_data:
                series_data[series_name] = []
            
            series_data[series_name].append({
                'timestamp': item['time'],
                'value': item['backlog'],
                'metadata': {
                    'queue': item['queue'],
                    'consumer_group': item.get('group'),
                    'published': item.get('published'),
                    'delivered': item.get('delivered'),
                    'pending': item.get('pending')
                }
            })
        
        # 转换为时间序列格式
        for series_name, data_points in series_data.items():
            monitoring_data['series'].append({
                'name': series_name,
                'data_points': data_points,
                'unit': 'tasks'
            })
        
        return MonitoringResponse(data=monitoring_data)
        
    except Exception as e:
        logger.error(f"获取积压趋势失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.get("/queue-flow-rates/{queue_name}", response_model=MonitoringResponse)
@cache_result(**CACHE_CONFIGS['monitoring_data'])
async def get_queue_flow_rates(
    queue_name: str,
    namespace: str = Depends(get_validated_namespace),
    time_params: dict = Depends(validate_time_range),
    granularity: Optional[str] = Query(None, description="数据粒度"),
    redis_client: redis.Redis = Depends(get_redis_client),
    pg_session: AsyncSession = Depends(get_pg_connection),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取队列流量速率"""
    metrics.start(namespace, f"GET /monitoring/queue-flow-rates/{queue_name}")
    print(f'{get_queue_flow_rates=}')
    try:
        # 使用现有的data_access来获取流量速率
        from ...data_access import JetTaskDataAccess
        
        data_access = JetTaskDataAccess()
        
        flow_data, returned_granularity = await data_access.fetch_queue_flow_rates(
            queue_name=queue_name,
            start_time=time_params.get('start_time'),
            end_time=time_params.get('end_time'),
            filters=[]
        )
        
        # 转换为监控响应格式
        series_data = {
            'enqueued': [],
            'started': [],
            'completed': []
        }
        
        for item in flow_data:
            timestamp = item['time']
            series_data['enqueued'].append({
                'timestamp': timestamp,
                'value': item.get('enqueued', 0),
                'metadata': {'queue': queue_name, 'type': 'enqueued'}
            })
            series_data['started'].append({
                'timestamp': timestamp,
                'value': item.get('started', 0),
                'metadata': {'queue': queue_name, 'type': 'started'}
            })
            series_data['completed'].append({
                'timestamp': timestamp,
                'value': item.get('completed', 0),
                'metadata': {'queue': queue_name, 'type': 'completed'}
            })
        
        monitoring_data = {
            'series': [
                {
                    'name': 'Enqueued',
                    'data_points': series_data['enqueued'],
                    'unit': 'tasks/min'
                },
                {
                    'name': 'Started',
                    'data_points': series_data['started'],
                    'unit': 'tasks/min'
                },
                {
                    'name': 'Completed',
                    'data_points': series_data['completed'],
                    'unit': 'tasks/min'
                }
            ],
            'granularity': returned_granularity,
            'time_range': {
                'start': time_params.get('start_time'),
                'end': time_params.get('end_time')
            }
        }
        
        return MonitoringResponse(data=monitoring_data)
        
    except Exception as e:
        logger.error(f"获取队列流量速率失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.get("/system-health")
async def get_system_health(
    namespace: str = Depends(get_validated_namespace),
    redis_client: redis.Redis = Depends(get_redis_client),
    pg_session: AsyncSession = Depends(get_pg_connection),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取系统健康状态"""
    metrics.start(namespace, "GET /monitoring/system-health")
    
    try:
        health_data = {
            'status': 'healthy',
            'version': '1.0.0',
            'uptime': 3600.0,  # 示例运行时间
            'components': {},
            'metrics': {}
        }
        
        # 检查Redis连接
        try:
            await redis_client.ping()
            health_data['components']['redis'] = 'healthy'
        except Exception as e:
            health_data['components']['redis'] = f'unhealthy: {str(e)}'
            health_data['status'] = 'degraded'
        
        # 检查PostgreSQL连接
        try:
            result = await pg_session.execute("SELECT 1")
            health_data['components']['postgresql'] = 'healthy'
        except Exception as e:
            health_data['components']['postgresql'] = f'unhealthy: {str(e)}'
            health_data['status'] = 'degraded'
        
        # 获取系统指标
        try:
            import psutil
            import time
            
            health_data['metrics'] = {
                'cpu_usage': psutil.cpu_percent(interval=1),
                'memory_usage': psutil.virtual_memory().percent,
                'disk_usage': psutil.disk_usage('/').percent,
                'process_count': len(psutil.pids()),
                'uptime': time.time() - psutil.boot_time()
            }
        except ImportError:
            health_data['metrics'] = {
                'note': 'psutil not available for system metrics'
            }
        
        return BaseResponse(data=health_data)
        
    except Exception as e:
        logger.error(f"获取系统健康状态失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.post("/analytics", response_model=AnalyticsResponse)
@cache_result(ttl=600, key_prefix="analytics")  # 10分钟缓存
async def get_analytics_data(
    request: AnalyticsRequest,
    namespace: str = Depends(get_validated_namespace),
    pg_session: AsyncSession = Depends(get_pg_connection),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取分析数据"""
    metrics.start(namespace, "POST /monitoring/analytics")
    
    try:
        analysis_type = request.analysis_type.lower()
        
        if analysis_type == "queue_performance":
            # 队列性能分析
            analytics_data = await _analyze_queue_performance(
                pg_session, request.dimensions, request.metrics,
                request.start_time, request.end_time
            )
        
        elif analysis_type == "task_distribution":
            # 任务分布分析
            analytics_data = await _analyze_task_distribution(
                pg_session, request.dimensions, request.metrics,
                request.start_time, request.end_time
            )
        
        elif analysis_type == "error_patterns":
            # 错误模式分析
            analytics_data = await _analyze_error_patterns(
                pg_session, request.dimensions, request.metrics,
                request.start_time, request.end_time
            )
        
        elif analysis_type == "resource_utilization":
            # 资源利用率分析
            analytics_data = await _analyze_resource_utilization(
                pg_session, request.dimensions, request.metrics,
                request.start_time, request.end_time
            )
        
        else:
            raise HTTPException(
                status_code=400,
                detail=f"Unsupported analysis type: {analysis_type}"
            )
        
        return AnalyticsResponse(data=analytics_data)
        
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取分析数据失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.get("/alerts/active")
async def get_active_alerts(
    namespace: str = Depends(get_validated_namespace),
    severity: Optional[str] = Query(None, description="告警级别筛选"),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取活跃告警"""
    metrics.start(namespace, "GET /monitoring/alerts/active")
    
    try:
        # 这里需要实现告警查询逻辑
        # 1. 从告警存储中获取活跃告警
        # 2. 根据严重程度筛选
        # 3. 返回告警详情
        
        # 暂时返回模拟数据
        alerts = [
            {
                "id": "alert_001",
                "rule_name": "High Queue Backlog",
                "severity": "warning",
                "status": "firing",
                "queue_name": "shared_queue",
                "trigger_value": 1200,
                "threshold": 1000,
                "started_at": "2025-09-08T12:00:00Z",
                "description": "Queue backlog exceeds threshold"
            },
            {
                "id": "alert_002", 
                "rule_name": "High Error Rate",
                "severity": "critical",
                "status": "firing",
                "queue_name": "priority_queue",
                "trigger_value": 0.15,
                "threshold": 0.10,
                "started_at": "2025-09-08T11:30:00Z",
                "description": "Error rate exceeds 10%"
            }
        ]
        
        # 应用严重程度筛选
        if severity:
            alerts = [alert for alert in alerts if alert['severity'] == severity]
        
        return BaseResponse(
            data={
                "alerts": alerts,
                "total_count": len(alerts),
                "counts_by_severity": {
                    "critical": len([a for a in alerts if a['severity'] == 'critical']),
                    "warning": len([a for a in alerts if a['severity'] == 'warning']),
                    "info": len([a for a in alerts if a['severity'] == 'info'])
                }
            }
        )
        
    except Exception as e:
        logger.error(f"获取活跃告警失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


# 辅助分析函数

async def _analyze_queue_performance(pg_session, dimensions, metrics, start_time, end_time):
    """分析队列性能"""
    # 这里实现队列性能分析逻辑
    return {
        "chart_data": [
            {"queue": "shared_queue", "avg_processing_time": 12.5, "throughput": 234.5},
            {"queue": "priority_queue", "avg_processing_time": 8.2, "throughput": 156.3}
        ],
        "summary": {
            "total_queues": 2,
            "avg_processing_time": 10.35,
            "total_throughput": 390.8
        },
        "insights": [
            "shared_queue has higher processing time but better throughput",
            "priority_queue shows good processing efficiency"
        ]
    }


async def _analyze_task_distribution(pg_session, dimensions, metrics, start_time, end_time):
    """分析任务分布"""
    # 这里实现任务分布分析逻辑
    return {
        "chart_data": [
            {"hour": 0, "task_count": 45, "queue": "shared_queue"},
            {"hour": 1, "task_count": 52, "queue": "shared_queue"},
            {"hour": 2, "task_count": 38, "queue": "shared_queue"}
        ],
        "summary": {
            "total_tasks": 135,
            "peak_hour": 1,
            "lowest_hour": 2
        },
        "insights": [
            "Task load is relatively stable with slight peak at hour 1"
        ]
    }


async def _analyze_error_patterns(pg_session, dimensions, metrics, start_time, end_time):
    """分析错误模式"""
    # 这里实现错误模式分析逻辑
    return {
        "chart_data": [
            {"error_type": "TimeoutError", "count": 25, "percentage": 45.5},
            {"error_type": "ConnectionError", "count": 18, "percentage": 32.7},
            {"error_type": "ValidationError", "count": 12, "percentage": 21.8}
        ],
        "summary": {
            "total_errors": 55,
            "most_common_error": "TimeoutError",
            "error_rate": 0.044
        },
        "insights": [
            "TimeoutError is the most common error type",
            "Consider increasing timeout values for better reliability"
        ]
    }


async def _analyze_resource_utilization(pg_session, dimensions, metrics, start_time, end_time):
    """分析资源利用率"""
    # 这里实现资源利用率分析逻辑
    return {
        "chart_data": [
            {"timestamp": "2025-09-08T12:00:00Z", "cpu": 45.2, "memory": 62.1, "disk": 23.4},
            {"timestamp": "2025-09-08T12:15:00Z", "cpu": 52.8, "memory": 65.3, "disk": 23.5}
        ],
        "summary": {
            "avg_cpu": 49.0,
            "avg_memory": 63.7,
            "avg_disk": 23.45
        },
        "insights": [
            "CPU utilization is moderate",
            "Memory usage is within acceptable range",
            "Disk usage is low"
        ]
    }


"""
任务中心客户端 - JetTask App使用的客户端库
"""
import os
import aiohttp
import asyncio
from typing import Optional, Dict, Any
from .models.namespace import TaskCenterConfig
import logging

logger = logging.getLogger(__name__)


class TaskCenterClient:
    """任务中心客户端"""
    
    def __init__(self, task_center_url: Optional[str] = None):
        """
        初始化任务中心客户端
        
        Args:
            task_center_url: 任务中心配置URL，支持两种格式:
                - HTTP格式: http://localhost:8001/api/namespaces/{namespace_id}
                - 旧格式: taskcenter://namespace/{namespace_id} (向后兼容)
        """
        self.task_center_url = task_center_url
        self._session: Optional[aiohttp.ClientSession] = None
        self._cached_config: Optional[Dict[str, Any]] = None
        self._namespace_id: Optional[str] = None
        self._namespace_name: Optional[str] = None  # 添加命名空间名称
        
        # 解析URL
        self._config_url = task_center_url  # 用于获取配置的URL
        if task_center_url:
            if task_center_url.startswith("http://") or task_center_url.startswith("https://"):
                # HTTP格式，支持新格式: http://localhost:8001/api/namespaces/{name}
                import re
                # 匹配新格式：使用名称而非ID
                match = re.search(r'/namespaces/([^/]+)$', task_center_url)
                if match:
                    self._namespace_name = match.group(1)
                    # 新格式直接使用URL，不需要添加/config
            elif task_center_url.startswith("taskcenter://"):
                # 旧格式，转换为HTTP格式
                parts = task_center_url.replace("taskcenter://", "").split("/")
                if len(parts) >= 2 and parts[0] == "namespace":
                    # 旧格式使用的是ID，转换为按名称查找
                    self._namespace_name = parts[1]  # 假设传入的也是名称
                    base_url = os.getenv("TASK_CENTER_BASE_URL", "http://localhost:8001")
                    self._config_url = f"{base_url}/api/v1/namespaces/{self._namespace_name}"
        
    @property
    def is_enabled(self) -> bool:
        """是否启用任务中心"""
        return self.task_center_url is not None
    
    @property
    def namespace_id(self) -> Optional[str]:
        """获取命名空间ID"""
        return self._namespace_id
    
    @property
    def namespace_prefix(self) -> str:
        """获取Redis key前缀"""
        # 优先使用namespace_name，不再添加tc:前缀
        if self._namespace_name:
            return self._namespace_name
        # 如果配置已加载，使用其中的名称
        elif self._cached_config and self._cached_config.get('namespace_name'):
            return self._cached_config['namespace_name']
        # 默认前缀
        return "jettask"
    
    async def _get_session(self) -> aiohttp.ClientSession:
        """获取HTTP会话"""
        if self._session is None or self._session.closed:
            self._session = aiohttp.ClientSession()
        return self._session
    
    async def get_config(self) -> Optional[Dict[str, Any]]:
        """
        从任务中心获取配置
        
        Returns:
            包含redis_config和pg_config的字典
        """
        if not self.is_enabled:
            return None
        
        # 如果已缓存，直接返回
        if self._cached_config:
            return self._cached_config
            
        try:
            session = await self._get_session()
            # 使用配置URL（可能带有/config后缀）
            async with session.get(self._config_url) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    self._namespace_name = data.get('name')  # 保存命名空间名称
                    self._cached_config = {
                        'redis_config': data.get('redis_config'),
                        'pg_config': data.get('pg_config'),
                        'namespace_name': data.get('name'),
                        'namespace_id': self._namespace_id,
                        'version': data.get('version', 1)  # 添加版本号
                    }
                    return self._cached_config
                else:
                    logger.error(f"Failed to get config from task center: {resp.status}")
                    return None
        except Exception as e:
            logger.error(f"Error getting config from task center: {e}")
            return None
    
    async def get_task_result(self, task_id: str) -> Optional[Dict[str, Any]]:
        """
        从任务中心获取任务结果（当Redis中不存在时）
        
        Args:
            task_id: 任务ID
            
        Returns:
            任务结果字典
        """
        if not self.is_enabled or not self._namespace_id:
            return None
            
        try:
            session = await self._get_session()
            # 从配置URL推导出任务结果URL
            base_url = self.task_center_url.replace(f"/namespace/{self._namespace_id}/config", "")
            url = f"{base_url}/namespace/{self._namespace_id}/task/{task_id}/result"
            async with session.get(url) as resp:
                if resp.status == 200:
                    return await resp.json()
                elif resp.status == 404:
                    return None
                else:
                    logger.error(f"Failed to get task result from task center: {resp.status}")
                    return None
        except Exception as e:
            logger.error(f"Error getting task result from task center: {e}")
            return None
    
    
    async def close(self):
        """关闭客户端"""
        if self._session:
            await self._session.close()


#!/usr/bin/env python
"""多命名空间消费者管理器

为每个命名空间启动独立的pg_consumer进程
支持动态添加/移除命名空间
"""

import asyncio
import logging
import multiprocessing as mp
import signal
import sys
import os
from typing import Dict, Optional, Set
from datetime import datetime

# 添加项目路径
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from jettask.pg_consumer import PostgreSQLConsumer
from jettask.webui_config import PostgreSQLConfig, RedisConfig
from jettask.core.consumer_manager import ConsumerStrategy
from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.orm import sessionmaker
from sqlalchemy import text

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(process)d - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)


class NamespaceConsumerProcess:
    """单个命名空间的消费进程"""
    
    def __init__(self, namespace_info: dict):
        """
        Args:
            namespace_info: 命名空间配置信息，包含:
                - id: 命名空间ID
                - name: 命名空间名称
                - redis_config: Redis配置
                - pg_config: PostgreSQL配置
                - redis_prefix: Redis键前缀
        """
        self.namespace_info = namespace_info
        self.process: Optional[mp.Process] = None
        
    def start(self):
        """启动进程"""
        if self.process and self.process.is_alive():
            logger.warning(f"命名空间 {self.namespace_info['name']} 的消费进程已在运行")
            return
            
        self.process = mp.Process(
            target=self._run_consumer,
            name=f"consumer_{self.namespace_info['name']}"
        )
        self.process.daemon = False
        self.process.start()
        logger.info(f"启动命名空间 {self.namespace_info['name']} 的消费进程, PID: {self.process.pid}")
        
    def stop(self):
        """停止进程"""
        if self.process and self.process.is_alive():
            logger.info(f"停止命名空间 {self.namespace_info['name']} 的消费进程")
            self.process.terminate()
            self.process.join(timeout=10)
            if self.process.is_alive():
                logger.warning(f"强制停止命名空间 {self.namespace_info['name']} 的消费进程")
                self.process.kill()
                self.process.join()
                
    def is_alive(self) -> bool:
        """检查进程是否存活"""
        return self.process and self.process.is_alive()
        
    def _run_consumer(self):
        """在子进程中运行消费者"""
        # 设置信号处理
        signal.signal(signal.SIGTERM, self._signal_handler)
        signal.signal(signal.SIGINT, self._signal_handler)
        
        # 创建新的事件循环
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        
        try:
            loop.run_until_complete(self._async_run())
        except KeyboardInterrupt:
            logger.info(f"命名空间 {self.namespace_info['name']} 的消费进程收到中断信号")
        except Exception as e:
            logger.error(f"命名空间 {self.namespace_info['name']} 的消费进程异常退出: {e}", exc_info=True)
        finally:
            loop.close()
            
    async def _async_run(self):
        """异步运行消费者"""
        namespace_name = self.namespace_info.get('name', 'unknown')
        logger.info(f"开始初始化命名空间 {namespace_name} 的消费者")
        
        try:
            # 创建配置
            pg_config_data = self.namespace_info.get('pg_config', {})
            
            # 如果包含 url 字段，使用 from_url 方法
            if 'url' in pg_config_data:
                pg_config = PostgreSQLConfig.from_url(pg_config_data['url'])
            else:
                # 移除不支持的字段
                valid_fields = {'host', 'port', 'database', 'user', 'password'}
                filtered_config = {k: v for k, v in pg_config_data.items() if k in valid_fields}
                pg_config = PostgreSQLConfig(**filtered_config)
            
            redis_config_data = self.namespace_info.get('redis_config', {})
            
            # 如果包含 url 字段，解析它
            if 'url' in redis_config_data:
                from urllib.parse import urlparse
                parsed = urlparse(redis_config_data['url'])
                redis_config = RedisConfig(
                    host=parsed.hostname or 'localhost',
                    port=parsed.port or 6379,
                    db=int(parsed.path.lstrip('/')) if parsed.path and parsed.path != '/' else 0,
                    password=parsed.password
                )
            else:
                # 移除不支持的字段
                valid_fields = {'host', 'port', 'db', 'password'}
                filtered_config = {k: v for k, v in redis_config_data.items() if k in valid_fields}
                redis_config = RedisConfig(**filtered_config)
            
            
            # 创建消费者实例
            consumer = PostgreSQLConsumer(
                pg_config=pg_config,
                redis_config=redis_config,
                prefix=self.namespace_info.get('redis_prefix', 'jettask'),
                namespace_id=self.namespace_info.get('id'),
                namespace_name=self.namespace_info.get('name'),
                consumer_strategy=ConsumerStrategy.HEARTBEAT
            )
            
            logger.info(f"命名空间 {namespace_name} 的消费者实例创建成功，准备启动")
            
            # 启动消费者
            await consumer.start()
            logger.info(f"命名空间 {namespace_name} 的消费者已启动，进入运行状态")
            
            # 保持运行直到收到停止信号
            try:
                while True:
                    await asyncio.sleep(10)  # 每10秒检查一次
                    # 可以在这里添加健康检查逻辑
            except asyncio.CancelledError:
                logger.info(f"命名空间 {namespace_name} 的消费者收到取消信号")
                await consumer.stop()
                raise
                
        except Exception as e:
            logger.error(f"命名空间 {namespace_name} 的消费者启动失败: {e}", exc_info=True)
            raise
            
    def _signal_handler(self, signum, frame):
        """信号处理器"""
        logger.info(f"命名空间 {self.namespace_info['name']} 的消费进程收到信号 {signum}")
        sys.exit(0)


class MultiNamespaceConsumerManager:
    """多命名空间消费者管理器"""
    
    def __init__(self, 
                 task_center_url: str,
                 namespace_check_interval: int = 60):
        """
        初始化管理器
        
        Args:
            task_center_url: 任务中心的URL（必需，从中获取数据库配置）
            namespace_check_interval: 命名空间检测间隔（秒）
        """
        self.consumers: Dict[str, NamespaceConsumerProcess] = {}
        self.running = False
        self.task_center_url = task_center_url.rstrip('/')
        self.namespace_check_interval = namespace_check_interval
        self.task_center_db_url = None
        
        logger.info(f"命名空间检测间隔设置为: {self.namespace_check_interval} 秒")
        self.known_namespaces: Set[str] = set()  # 已知的命名空间集合
            
    async def start(self, namespace_names: Optional[Set[str]] = None):
        """启动管理器
        
        Args:
            namespace_names: 要启动的命名空间名称集合，None表示启动所有命名空间
        """
        self.running = True
        logger.info("启动多命名空间消费者管理器")
        
        # 从任务中心获取数据库配置
        await self._init_database_config()
        
        # 获取命名空间配置
        namespaces = await self._fetch_namespaces(namespace_names)
        
        # 启动每个命名空间的消费者
        for ns_info in namespaces:
            try:
                self._start_namespace_consumer(ns_info)
                self.known_namespaces.add(ns_info['name'])
            except Exception as e:
                logger.error(f"启动命名空间 {ns_info['name']} 的消费者失败: {e}")
                
        # 创建并发任务：健康检查和命名空间检测
        try:
            health_check_task = asyncio.create_task(self._health_check_loop())
            namespace_check_task = asyncio.create_task(self._namespace_check_loop())
            
            # 等待任一任务完成（或出错）
            done, pending = await asyncio.wait(
                [health_check_task, namespace_check_task],
                return_when=asyncio.FIRST_EXCEPTION
            )
            
            # 取消所有未完成的任务
            for task in pending:
                task.cancel()
                
        except KeyboardInterrupt:
            logger.info("收到中断信号，停止管理器")
        finally:
            await self.stop()
            
    async def stop(self):
        """停止所有消费者"""
        self.running = False
        logger.info("停止所有命名空间消费者")
        
        for name, consumer in self.consumers.items():
            consumer.stop()
            
        self.consumers.clear()
        
    def _start_namespace_consumer(self, namespace_info: dict):
        """启动单个命名空间的消费者"""
        name = namespace_info['name']
        
        # 如果已存在，先停止
        if name in self.consumers:
            self.consumers[name].stop()
            
        # 创建并启动新进程
        consumer = NamespaceConsumerProcess(namespace_info)
        consumer.start()
        self.consumers[name] = consumer
        
    async def _init_database_config(self):
        """从任务中心获取数据库配置"""
        import aiohttp
        import json
        
        # 根据 URL 格式判断如何获取配置
        base_url = self.task_center_url
        
        # 如果是单命名空间URL，提取基础URL
        if '/api/namespaces/' in base_url:
            # 从 http://localhost:8001/api/namespaces/default 提取 http://localhost:8001
            base_url = base_url.split('/api/namespaces/')[0]
        elif '/api/v1/namespaces/' in base_url:
            # 从 http://localhost:8001/api/v1/namespaces/default 提取 http://localhost:8001
            base_url = base_url.split('/api/v1/namespaces/')[0]
        
        # 获取API服务的配置端点
        config_url = f"{base_url}/api/config"
        
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(config_url) as response:
                    if response.status == 200:
                        config = await response.json()
                        # 从配置中提取数据库URL
                        if 'database' in config:
                            db_config = config['database']
                            host = db_config.get('host', 'localhost')
                            port = db_config.get('port', 5432)
                            user = db_config.get('user', 'jettask')
                            password = db_config.get('password', '123456')
                            database = db_config.get('database', 'jettask')
                            self.task_center_db_url = f"postgresql+asyncpg://{user}:{password}@{host}:{port}/{database}"
                        else:
                            # 如果没有配置端点，使用默认值
                            logger.warning(f"无法从任务中心获取数据库配置，使用默认配置")
                            self.task_center_db_url = "postgresql+asyncpg://jettask:123456@localhost:5432/jettask"
                    else:
                        # API端点不存在，使用默认值
                        logger.warning(f"任务中心配置端点返回错误: {response.status}，使用默认配置")
                        self.task_center_db_url = "postgresql+asyncpg://jettask:123456@localhost:5432/jettask"
        except Exception as e:
            logger.warning(f"无法连接到任务中心获取配置: {e}，使用默认配置")
            # 使用默认配置
            self.task_center_db_url = "postgresql+asyncpg://jettask:123456@localhost:5432/jettask"
        
        logger.info(f"数据库配置已初始化")
        
    async def _fetch_namespaces(self, namespace_names: Optional[Set[str]] = None) -> list:
        """从任务中心数据库获取命名空间配置"""
        engine = create_async_engine(self.task_center_db_url, echo=False)
        AsyncSessionLocal = sessionmaker(engine, class_=AsyncSession, expire_on_commit=False)
        
        namespaces = []
        
        async with AsyncSessionLocal() as session:
            try:
                # 查询命名空间
                if namespace_names:
                    query = text("""
                        SELECT id, name, redis_config, pg_config 
                        FROM namespaces 
                        WHERE name = ANY(:names)
                    """)
                    result = await session.execute(query, {'names': list(namespace_names)})
                else:
                    query = text("""
                        SELECT id, name, redis_config, pg_config 
                        FROM namespaces
                    """)
                    result = await session.execute(query)
                    
                for row in result:
                    # 直接使用 redis_config 和 pg_config
                    redis_cfg = row.redis_config if row.redis_config else {}
                    pg_cfg = row.pg_config if row.pg_config else {}
                    
                    # 处理 Redis 密码：如果为空字符串或 "None"，设置为 None
                    redis_password = redis_cfg.get('password')
                    if redis_password in ['', 'None', 'null']:
                        redis_password = None
                    
                    # 确定 Redis 前缀：
                    # 1. 优先使用配置中的 prefix
                    # 2. 其次使用命名空间名
                    # 3. 最后使用 'jettask' 作为默认值
                    redis_prefix = redis_cfg.get('prefix')
                    if not redis_prefix:
                        # 使用命名空间名作为前缀，这样不同命名空间的数据在 Redis 中是隔离的
                        redis_prefix = row.name
                    
                    logger.info(f"命名空间 {row.name} 使用 Redis 前缀: {redis_prefix}")
                    
                    namespaces.append({
                        'id': row.id,
                        'name': row.name,
                        'redis_config': {
                            'host': redis_cfg.get('host', 'localhost'),
                            'port': redis_cfg.get('port', 6379),
                            'db': redis_cfg.get('db', 0),
                            'password': redis_password
                        },
                        'pg_config': {
                            'host': pg_cfg.get('host', 'localhost'),
                            'port': pg_cfg.get('port', 5432),
                            'database': pg_cfg.get('database', 'jettask'),
                            'user': pg_cfg.get('user', 'jettask'),
                            'password': pg_cfg.get('password', '123456')
                        },
                        'redis_prefix': redis_prefix
                    })
                        
            except Exception as e:
                logger.error(f"查询命名空间失败: {e}")
                # 如果查询失败，使用默认配置
                if not namespace_names or 'default' in namespace_names:
                    namespaces.append(self._get_default_namespace())
                    
        await engine.dispose()
        return namespaces
        
    def _parse_connection_config(self, connection_url: str) -> Optional[dict]:
        """解析连接配置字符串"""
        try:
            import json
            config = json.loads(connection_url)
            
            # 提取Redis配置
            redis_config = {}
            if 'redis' in config:
                redis_info = config['redis']
                redis_config = {
                    'host': redis_info.get('host', 'localhost'),
                    'port': redis_info.get('port', 6379),
                    'db': redis_info.get('db', 0),
                    'password': redis_info.get('password')
                }
                
            # 提取PostgreSQL配置
            pg_config = {}
            if 'postgres' in config:
                pg_info = config['postgres']
                pg_config = {
                    'host': pg_info.get('host', 'localhost'),
                    'port': pg_info.get('port', 5432),
                    'database': pg_info.get('database', 'jettask'),
                    'user': pg_info.get('user', 'jettask'),
                    'password': pg_info.get('password', '123456')
                }
                
            return {
                'redis_config': redis_config,
                'pg_config': pg_config,
                'redis_prefix': config.get('prefix', 'jettask')
            }
            
        except Exception as e:
            logger.error(f"解析连接配置失败: {e}")
            return None
            
    def _get_default_namespace(self) -> dict:
        """获取默认命名空间配置"""
        return {
            'id': 'default',
            'name': 'default',
            'redis_config': {
                'host': os.environ.get('REDIS_HOST', 'localhost'),
                'port': int(os.environ.get('REDIS_PORT', 6379)),
                'db': int(os.environ.get('REDIS_DB', 0)),
                'password': os.environ.get('REDIS_PASSWORD')
            },
            'pg_config': {
                'host': os.environ.get('POSTGRES_HOST', 'localhost'),
                'port': int(os.environ.get('POSTGRES_PORT', 5432)),
                'database': os.environ.get('POSTGRES_DB', 'jettask'),
                'user': os.environ.get('POSTGRES_USER', 'jettask'),
                'password': os.environ.get('POSTGRES_PASSWORD', '123456')
            },
            'redis_prefix': os.environ.get('REDIS_PREFIX', 'default')  # 使用 'default' 作为默认命名空间的前缀
        }
        
    async def _health_check_loop(self):
        """健康检查循环"""
        while self.running:
            try:
                await self._health_check()
                await asyncio.sleep(30)  # 每30秒检查一次
            except Exception as e:
                logger.error(f"健康检查出错: {e}", exc_info=True)
                await asyncio.sleep(30)
                
    async def _namespace_check_loop(self):
        """命名空间检测循环"""
        # 首次启动后快速检测一次（5秒后）
        first_check_delay = min(5, self.namespace_check_interval)
        await asyncio.sleep(first_check_delay)
        
        while self.running:
            try:
                await self._check_new_namespaces()
                await asyncio.sleep(self.namespace_check_interval)
            except Exception as e:
                logger.error(f"命名空间检测出错: {e}", exc_info=True)
                await asyncio.sleep(self.namespace_check_interval)
                
    async def _check_new_namespaces(self):
        """检查是否有新的命名空间"""
        logger.debug("开始检查新命名空间...")
        
        # 获取所有活跃的命名空间
        current_namespaces = await self._fetch_namespaces()
        current_names = {ns['name'] for ns in current_namespaces}
        
        # 找出新增的命名空间
        new_names = current_names - self.known_namespaces
        if new_names:
            logger.info(f"发现新的命名空间: {new_names}")
            
            # 为新命名空间启动消费者
            for ns_info in current_namespaces:
                if ns_info['name'] in new_names:
                    try:
                        logger.info(f"为新命名空间 {ns_info['name']} 启动消费者")
                        self._start_namespace_consumer(ns_info)
                        self.known_namespaces.add(ns_info['name'])
                    except Exception as e:
                        logger.error(f"启动新命名空间 {ns_info['name']} 的消费者失败: {e}")
        
        # 找出被删除的命名空间
        removed_names = self.known_namespaces - current_names
        if removed_names:
            logger.info(f"发现被删除的命名空间: {removed_names}")
            
            # 停止被删除命名空间的消费者
            for name in removed_names:
                if name in self.consumers:
                    logger.info(f"停止已删除命名空间 {name} 的消费者")
                    self.consumers[name].stop()
                    del self.consumers[name]
                self.known_namespaces.discard(name)
                
    async def _health_check(self):
        """检查消费进程健康状态"""
        for name, consumer in list(self.consumers.items()):
            if not consumer.is_alive():
                logger.warning(f"命名空间 {name} 的消费进程已停止，尝试重启")
                
                # 重新获取配置并重启
                namespaces = await self._fetch_namespaces({name})
                if namespaces:
                    self._start_namespace_consumer(namespaces[0])
                else:
                    logger.error(f"无法获取命名空间 {name} 的配置，移除该消费者")
                    del self.consumers[name]
                    self.known_namespaces.discard(name)


async def main():
    """主函数"""
    import argparse
    
    parser = argparse.ArgumentParser(description='多命名空间PostgreSQL消费者')
    parser.add_argument(
        '--namespaces',
        nargs='*',
        help='要启动的命名空间列表，不指定则启动所有命名空间'
    )
    
    args = parser.parse_args()
    
    # 创建管理器
    manager = MultiNamespaceConsumerManager()
    
    # 设置信号处理
    def signal_handler(signum, frame):
        logger.info(f"收到信号 {signum}，准备退出")
        asyncio.create_task(manager.stop())
        
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    
    # 启动管理器
    namespace_set = set(args.namespaces) if args.namespaces else None
    await manager.start(namespace_set)


if __name__ == '__main__':
    asyncio.run(main())


"""
Response models for JetTask WebUI Backend
"""
from typing import List, Dict, Any, Optional, Generic, TypeVar
from datetime import datetime
from pydantic import BaseModel, Field

T = TypeVar('T')


class BaseResponse(BaseModel, Generic[T]):
    """基础响应模型"""
    success: bool = Field(default=True, description="请求是否成功")
    message: Optional[str] = Field(default=None, description="响应消息")
    data: Optional[T] = Field(default=None, description="响应数据")
    timestamp: datetime = Field(default_factory=datetime.now, description="响应时间戳")


class PaginatedResponse(BaseResponse[List[T]]):
    """分页响应模型"""
    total: int = Field(description="总记录数")
    page: int = Field(description="当前页码")
    page_size: int = Field(description="每页大小")
    total_pages: int = Field(description="总页数")
    
    @classmethod
    def create(
        cls,
        data: List[T],
        total: int,
        page: int,
        page_size: int,
        message: Optional[str] = None
    ):
        """创建分页响应"""
        total_pages = (total + page_size - 1) // page_size
        return cls(
            data=data,
            total=total,
            page=page,
            page_size=page_size,
            total_pages=total_pages,
            message=message
        )


class ErrorResponse(BaseModel):
    """错误响应模型"""
    success: bool = Field(default=False)
    error_code: str = Field(description="错误码")
    message: str = Field(description="错误信息")
    details: Optional[Dict[str, Any]] = Field(default=None, description="错误详情")
    timestamp: datetime = Field(default_factory=datetime.now)


# 队列相关响应模型
class QueueInfo(BaseModel):
    """队列基本信息"""
    name: str = Field(description="队列名称")
    namespace: str = Field(description="命名空间")
    priority: Optional[int] = Field(default=None, description="优先级")
    pending_count: int = Field(default=0, description="待处理任务数")
    running_count: int = Field(default=0, description="运行中任务数") 
    completed_count: int = Field(default=0, description="已完成任务数")
    failed_count: int = Field(default=0, description="失败任务数")
    last_activity: Optional[datetime] = Field(default=None, description="最后活动时间")


class QueueStats(BaseModel):
    """队列统计信息"""
    queue_info: QueueInfo
    consumer_groups: List[Dict[str, Any]] = Field(default_factory=list, description="消费者组信息")
    metrics: Dict[str, Any] = Field(default_factory=dict, description="性能指标")
    trends: Dict[str, Any] = Field(default_factory=dict, description="趋势数据")


class QueueListResponse(BaseResponse[List[QueueInfo]]):
    """队列列表响应"""
    pass


class QueueDetailResponse(BaseResponse[QueueStats]):
    """队列详情响应"""
    pass


# 任务相关响应模型
class TaskInfo(BaseModel):
    """任务基本信息"""
    id: str = Field(description="任务ID")
    stream_id: Optional[str] = Field(default=None, description="流ID")
    queue: str = Field(description="队列名称")
    task_name: Optional[str] = Field(default=None, description="任务名称")
    status: str = Field(description="任务状态")
    created_at: datetime = Field(description="创建时间")
    started_at: Optional[datetime] = Field(default=None, description="开始时间")
    completed_at: Optional[datetime] = Field(default=None, description="完成时间")
    duration: Optional[float] = Field(default=None, description="执行时长(秒)")
    worker_id: Optional[str] = Field(default=None, description="工作者ID")
    consumer_group: Optional[str] = Field(default=None, description="消费者组")
    priority: Optional[int] = Field(default=None, description="优先级")


class TaskDetail(TaskInfo):
    """任务详细信息"""
    task_data: Optional[Dict[str, Any]] = Field(default=None, description="任务数据")
    result: Optional[Dict[str, Any]] = Field(default=None, description="执行结果")
    error_info: Optional[Dict[str, Any]] = Field(default=None, description="错误信息")
    retry_count: int = Field(default=0, description="重试次数")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="元数据")


class TaskListResponse(PaginatedResponse[TaskInfo]):
    """任务列表响应"""
    pass


class TaskDetailResponse(BaseResponse[TaskDetail]):
    """任务详情响应"""
    pass


# 监控相关响应模型
class MetricPoint(BaseModel):
    """指标数据点"""
    timestamp: datetime = Field(description="时间戳")
    value: float = Field(description="指标值")
    metadata: Optional[Dict[str, Any]] = Field(default=None, description="元数据")


class TimeSeries(BaseModel):
    """时间序列数据"""
    name: str = Field(description="序列名称")
    data_points: List[MetricPoint] = Field(description="数据点")
    unit: Optional[str] = Field(default=None, description="单位")


class MonitoringMetrics(BaseModel):
    """监控指标"""
    series: List[TimeSeries] = Field(description="时间序列数据")
    granularity: str = Field(description="数据粒度")
    time_range: Dict[str, datetime] = Field(description="时间范围")


class MonitoringResponse(BaseResponse[MonitoringMetrics]):
    """监控数据响应"""
    pass


# 分析相关响应模型
class AnalyticsData(BaseModel):
    """分析数据"""
    chart_data: List[Dict[str, Any]] = Field(description="图表数据")
    summary: Dict[str, Any] = Field(description="汇总信息")
    insights: List[str] = Field(default_factory=list, description="分析洞察")


class AnalyticsResponse(BaseResponse[AnalyticsData]):
    """分析响应"""
    pass


# 定时任务相关响应模型
class ScheduledTaskInfo(BaseModel):
    """定时任务信息"""
    id: str = Field(description="任务ID")
    name: str = Field(description="任务名称")
    namespace: str = Field(description="命名空间")
    queue_name: str = Field(description="队列名称")
    task_name: str = Field(description="任务函数名")
    schedule_type: str = Field(description="调度类型")
    schedule_config: Dict[str, Any] = Field(description="调度配置")
    is_active: bool = Field(description="是否启用")
    created_at: datetime = Field(description="创建时间")
    updated_at: Optional[datetime] = Field(default=None, description="更新时间")
    last_run_at: Optional[datetime] = Field(default=None, description="最后运行时间")
    next_run_at: Optional[datetime] = Field(default=None, description="下次运行时间")
    run_count: int = Field(default=0, description="运行次数")
    success_count: int = Field(default=0, description="成功次数")
    failure_count: int = Field(default=0, description="失败次数")


class ScheduledTaskListResponse(PaginatedResponse[ScheduledTaskInfo]):
    """定时任务列表响应"""
    pass


class ScheduledTaskResponse(BaseResponse[ScheduledTaskInfo]):
    """定时任务响应"""
    pass


# 命名空间相关响应模型
class NamespaceInfo(BaseModel):
    """命名空间信息"""
    id: str = Field(description="命名空间ID")
    name: str = Field(description="命名空间名称")
    display_name: str = Field(description="显示名称")
    description: Optional[str] = Field(default=None, description="描述")
    redis_url: str = Field(description="Redis连接URL")
    pg_url: Optional[str] = Field(default=None, description="PostgreSQL连接URL")
    created_at: datetime = Field(description="创建时间")
    is_active: bool = Field(default=True, description="是否启用")
    queue_count: int = Field(default=0, description="队列数量")
    task_count: int = Field(default=0, description="任务数量")


class NamespaceListResponse(BaseResponse[List[NamespaceInfo]]):
    """命名空间列表响应"""
    pass


class NamespaceResponse(BaseResponse[NamespaceInfo]):
    """命名空间响应"""
    pass


# 系统状态响应模型
class SystemHealth(BaseModel):
    """系统健康状态"""
    status: str = Field(description="系统状态")
    version: str = Field(description="系统版本")
    uptime: float = Field(description="运行时间(秒)")
    components: Dict[str, str] = Field(description="组件状态")
    metrics: Dict[str, Any] = Field(description="系统指标")


class HealthResponse(BaseResponse[SystemHealth]):
    """健康检查响应"""
    pass


"""
Queue management API v1
"""
from typing import List, Optional
from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.ext.asyncio import AsyncSession
import redis.asyncio as redis

from dependencies import (
    get_validated_namespace, get_pg_connection, get_redis_client,
    get_namespace_connection, validate_page_params, validate_time_range,
    get_request_metrics, RequestMetrics
)
from models.requests import QueueListRequest, QueueMetricsRequest, QueueActionRequest
from models.responses import QueueListResponse, QueueDetailResponse, MonitoringResponse, BaseResponse
from core.cache import cache_result, CACHE_CONFIGS
from core.exceptions import QueueNotFoundError, ValidationError
from queue_stats_v2 import QueueStatsV2
import logging

logger = logging.getLogger(__name__)
router = APIRouter()


@router.get("", response_model=QueueListResponse)
@cache_result(**CACHE_CONFIGS['queue_stats'])
async def list_queues(
    namespace: str = Depends(get_validated_namespace),
    page: int = Query(1, ge=1, description="页码"),
    page_size: int = Query(20, ge=1, le=100, description="每页大小"),
    search: Optional[str] = Query(None, description="搜索关键词"),
    status: Optional[str] = Query(None, description="状态筛选"),
    include_stats: bool = Query(True, description="是否包含统计信息"),
    redis_client: redis.Redis = Depends(get_redis_client),
    pg_session: AsyncSession = Depends(get_pg_connection),
    connection = Depends(get_namespace_connection),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取队列列表"""
    metrics.start(namespace, "GET /queues")
    
    try:
        # 创建队列统计服务
        stats_service = QueueStatsV2(
            redis_client=redis_client,
            pg_session=pg_session,
            redis_prefix=connection.redis_prefix
        )
        
        # 获取队列统计数据
        queue_stats = await stats_service.get_queue_stats_grouped()
        
        # 提取基础队列名（去除优先级后缀）
        base_queues = {}
        for stat in queue_stats:
            base_name = get_base_queue_name(stat['queue_name'])
            if base_name not in base_queues:
                base_queues[base_name] = {
                    'name': base_name,
                    'namespace': namespace,
                    'priority': None,
                    'pending_count': 0,
                    'running_count': 0,
                    'completed_count': 0,
                    'failed_count': 0,
                    'last_activity': None
                }
            
            # 聚合统计数据
            queue_info = base_queues[base_name]
            queue_info['pending_count'] += stat.get('unprocessed_tasks', 0)
            queue_info['running_count'] += stat.get('pending_in_runs', 0)
            queue_info['completed_count'] += stat.get('success_count', 0)
            queue_info['failed_count'] += stat.get('error_count', 0)
            
            # 更新最后活动时间
            if stat.get('last_activity') and (
                not queue_info['last_activity'] or 
                stat['last_activity'] > queue_info['last_activity']
            ):
                queue_info['last_activity'] = stat['last_activity']
        
        # 转换为响应格式
        queue_list = list(base_queues.values())
        
        # 应用搜索筛选
        if search:
            queue_list = [q for q in queue_list if search.lower() in q['name'].lower()]
        
        # 应用状态筛选
        if status:
            if status == 'active':
                queue_list = [q for q in queue_list if q['pending_count'] > 0 or q['running_count'] > 0]
            elif status == 'idle':
                queue_list = [q for q in queue_list if q['pending_count'] == 0 and q['running_count'] == 0]
        
        # 分页
        total = len(queue_list)
        start = (page - 1) * page_size
        end = start + page_size
        paginated_queues = queue_list[start:end]
        
        return QueueListResponse.create(
            data=paginated_queues,
            total=total,
            page=page,
            page_size=page_size
        )
        
    except Exception as e:
        logger.error(f"获取队列列表失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.get("/{queue_name}", response_model=QueueDetailResponse)
@cache_result(**CACHE_CONFIGS['queue_stats'])
async def get_queue_detail(
    queue_name: str,
    namespace: str = Depends(get_validated_namespace),
    redis_client: redis.Redis = Depends(get_redis_client),
    pg_session: AsyncSession = Depends(get_pg_connection),
    connection = Depends(get_namespace_connection),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取队列详情"""
    metrics.start(namespace, f"GET /queues/{queue_name}")
    
    try:
        # 创建队列统计服务
        stats_service = QueueStatsV2(
            redis_client=redis_client,
            pg_session=pg_session,
            redis_prefix=connection.redis_prefix
        )
        
        # 获取队列的详细统计
        queue_stats = await stats_service.get_queue_stats_grouped()
        
        # 筛选指定队列的数据（包括优先级队列）
        queue_data = []
        for stat in queue_stats:
            base_name = get_base_queue_name(stat['queue_name'])
            if base_name == queue_name:
                queue_data.append(stat)
        
        if not queue_data:
            raise QueueNotFoundError(queue_name, namespace)
        
        # 聚合队列基本信息
        queue_info = {
            'name': queue_name,
            'namespace': namespace,
            'priority': None,
            'pending_count': sum(s.get('unprocessed_tasks', 0) for s in queue_data),
            'running_count': sum(s.get('pending_in_runs', 0) for s in queue_data),
            'completed_count': sum(s.get('success_count', 0) for s in queue_data),
            'failed_count': sum(s.get('error_count', 0) for s in queue_data),
            'last_activity': max((s.get('last_activity') for s in queue_data if s.get('last_activity')), default=None)
        }
        
        # 提取消费者组信息
        consumer_groups = []
        for stat in queue_data:
            if stat.get('group_name'):
                consumer_groups.append({
                    'name': stat['group_name'],
                    'queue_name': stat['queue_name'],
                    'pending_count': stat.get('pending_in_runs', 0),
                    'processed_count': stat.get('processed_tasks', 0),
                    'success_count': stat.get('success_count', 0),
                    'error_count': stat.get('error_count', 0),
                    'avg_execution_time': stat.get('avg_execution_time'),
                    'recent_completed': stat.get('recent_completed', 0)
                })
        
        # 构造响应数据
        queue_detail = {
            'queue_info': queue_info,
            'consumer_groups': consumer_groups,
            'metrics': {
                'total_tasks': queue_info['pending_count'] + queue_info['running_count'] + queue_info['completed_count'] + queue_info['failed_count'],
                'success_rate': queue_info['completed_count'] / max(queue_info['completed_count'] + queue_info['failed_count'], 1),
                'active_consumer_groups': len([cg for cg in consumer_groups if cg['pending_count'] > 0])
            },
            'trends': {}  # 可以后续添加趋势数据
        }
        
        return QueueDetailResponse(data=queue_detail)
        
    except QueueNotFoundError:
        raise
    except Exception as e:
        logger.error(f"获取队列详情失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.get("/{queue_name}/metrics", response_model=MonitoringResponse)
@cache_result(**CACHE_CONFIGS['monitoring_data'])
async def get_queue_metrics(
    queue_name: str,
    namespace: str = Depends(get_validated_namespace),
    time_params: dict = Depends(validate_time_range),
    metrics_types: str = Query("pending,processing,completed", description="指标类型，逗号分隔"),
    granularity: Optional[str] = Query(None, description="数据粒度"),
    include_consumer_groups: bool = Query(False, description="是否包含消费者组数据"),
    connection = Depends(get_namespace_connection),
    request_metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取队列监控指标"""
    request_metrics.start(namespace, f"GET /queues/{queue_name}/metrics")
    
    try:
        # 这里使用现有的积压监控API功能
        from ...queue_backlog_api import get_backlog_trend, BacklogTrendRequest
        
        # 构造请求参数
        backlog_request = BacklogTrendRequest(
            namespace=namespace,
            queues=[queue_name],
            time_range=time_params.get('time_range'),
            start_time=time_params.get('start_time'),
            end_time=time_params.get('end_time'),
            granularity=granularity,
            include_groups=include_consumer_groups
        )
        
        # 调用积压趋势API
        backlog_response = await get_backlog_trend(backlog_request)
        
        # 转换为监控响应格式
        monitoring_data = {
            'series': [],
            'granularity': backlog_response.granularity,
            'time_range': backlog_response.time_range
        }
        
        # 按系列分组数据
        series_data = {}
        for item in backlog_response.data:
            series_name = item.get('group') or queue_name
            if series_name not in series_data:
                series_data[series_name] = []
            
            series_data[series_name].append({
                'timestamp': item['time'],
                'value': item['backlog'],
                'metadata': {
                    'queue': item['queue'],
                    'consumer_group': item.get('group'),
                    'published': item.get('published'),
                    'delivered': item.get('delivered')
                }
            })
        
        # 转换为时间序列格式
        for series_name, data_points in series_data.items():
            monitoring_data['series'].append({
                'name': series_name,
                'data_points': data_points,
                'unit': 'tasks'
            })
        
        return MonitoringResponse(data=monitoring_data)
        
    except Exception as e:
        logger.error(f"获取队列指标失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        request_metrics.finish()


@router.post("/{queue_name}/actions", response_model=BaseResponse)
async def execute_queue_action(
    queue_name: str,
    action_request: QueueActionRequest,
    namespace: str = Depends(get_validated_namespace),
    redis_client: redis.Redis = Depends(get_redis_client),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """执行队列操作"""
    metrics.start(namespace, f"POST /queues/{queue_name}/actions")
    
    try:
        action = action_request.action.lower()
        parameters = action_request.parameters
        
        if action == "trim":
            # 裁剪队列
            max_length = parameters.get('max_length')
            if not max_length or max_length < 0:
                raise ValidationError("max_length parameter is required and must be >= 0")
            
            # 这里需要实现Redis Stream的XTRIM操作
            # 暂时返回模拟响应
            return BaseResponse(
                message=f"Queue {queue_name} trimmed to {max_length} messages"
            )
        
        elif action == "clear":
            # 清空队列
            # 这里需要实现清空队列的逻辑
            return BaseResponse(
                message=f"Queue {queue_name} cleared"
            )
        
        elif action == "pause":
            # 暂停队列
            # 这里需要实现暂停消费的逻辑
            return BaseResponse(
                message=f"Queue {queue_name} paused"
            )
        
        elif action == "resume":
            # 恢复队列
            # 这里需要实现恢复消费的逻辑
            return BaseResponse(
                message=f"Queue {queue_name} resumed"
            )
        
        else:
            raise ValidationError(f"Unsupported action: {action}")
            
    except ValidationError:
        raise
    except Exception as e:
        logger.error(f"执行队列操作失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


def get_base_queue_name(queue_name: str) -> str:
    """提取基础队列名（去除优先级后缀）"""
    if ':' in queue_name:
        parts = queue_name.rsplit(':', 1)
        if parts[-1].isdigit():
            return parts[0]
    return queue_name


"""
定时任务管理器 - 负责数据库CRUD操作
"""
import asyncio
import asyncpg
from typing import Optional, List, Dict, Any
from datetime import datetime, timedelta
import json

from .models import ScheduledTask, TaskExecutionHistory, TaskType, TaskStatus


class ScheduledTaskManager:
    """定时任务数据库管理器"""
    
    def __init__(self, app_or_db_url):
        """
        初始化管理器
        
        Args:
            app_or_db_url: Jettask应用实例或PostgreSQL连接URL字符串
        """
        # 支持两种初始化方式：传入app对象或直接传入db_url
        if isinstance(app_or_db_url, str):
            self.db_url = app_or_db_url
        else:
            # 从app对象获取pg_url
            self.db_url = app_or_db_url.pg_url
        
        # 将SQLAlchemy格式的URL转换为原生PostgreSQL URL
        # postgresql+asyncpg:// -> postgresql://
        if self.db_url and '+' in self.db_url:
            self.db_url = self.db_url.split('+')[0] + self.db_url[self.db_url.index('://'):]
        
        self.pool: Optional[asyncpg.Pool] = None
    
    async def connect(self):
        """建立数据库连接池"""
        if not self.pool:
            self.pool = await asyncpg.create_pool(
                self.db_url,
                min_size=2,
                max_size=10,
                command_timeout=60
            )
    
    async def disconnect(self):
        """关闭数据库连接池"""
        if self.pool:
            await self.pool.close()
            self.pool = None
    
    async def init_schema(self):
        """初始化数据库表结构（幂等操作）"""
        import os
        schema_path = os.path.join(os.path.dirname(__file__), 'schema.sql')
        
        with open(schema_path, 'r') as f:
            schema_sql = f.read()
        
        async with self.pool.acquire() as conn:
            # 使用事务，忽略已存在的对象错误
            try:
                await conn.execute(schema_sql)
            except Exception as e:
                if "already exists" in str(e):
                    # 表或索引已存在，这是正常的
                    pass
                else:
                    # 其他错误则重新抛出
                    raise
    
    # ==================== 任务CRUD操作 ====================
    
    async def create_task(self, task: ScheduledTask) -> ScheduledTask:
        """创建定时任务"""
        sql = """
            INSERT INTO scheduled_tasks (
                scheduler_id, task_name, task_type, queue_name, namespace,
                task_args, task_kwargs, cron_expression, interval_seconds,
                next_run_time, enabled, max_retries, retry_delay, timeout,
                priority, description, tags, metadata
            ) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17, $18)
            RETURNING *
        """
        
        async with self.pool.acquire() as conn:
            row = await conn.fetchrow(
                sql,
                task.scheduler_id,
                task.task_name,
                task.task_type.value,
                task.queue_name,
                task.namespace,  # 添加namespace
                json.dumps(task.task_args),
                json.dumps(task.task_kwargs),
                task.cron_expression,
                task.interval_seconds,
                task.next_run_time,
                task.enabled,
                task.max_retries,
                task.retry_delay,
                task.timeout,
                task.priority,
                task.description,
                json.dumps(task.tags),
                json.dumps(task.metadata)
            )
            
            return self._row_to_task(row)
    
    async def get_task(self, task_id: int) -> Optional[ScheduledTask]:
        """获取单个任务"""
        sql = "SELECT * FROM scheduled_tasks WHERE id = $1"
        
        async with self.pool.acquire() as conn:
            row = await conn.fetchrow(sql, task_id)
            if row:
                return self._row_to_task(row)
            return None
    
    async def get_task_by_scheduler_id(self, scheduler_id: str) -> Optional[ScheduledTask]:
        """通过scheduler_id获取任务"""
        sql = "SELECT * FROM scheduled_tasks WHERE scheduler_id = $1"
        
        async with self.pool.acquire() as conn:
            row = await conn.fetchrow(sql, scheduler_id)
            if row:
                return self._row_to_task(row)
            return None
    
    async def update_task(self, task: ScheduledTask) -> ScheduledTask:
        """更新任务"""
        sql = """
            UPDATE scheduled_tasks SET
                scheduler_id = $2,
                task_name = $3,
                task_type = $4,
                queue_name = $5,
                namespace = $6,
                task_args = $7,
                task_kwargs = $8,
                cron_expression = $9,
                interval_seconds = $10,
                next_run_time = $11,
                last_run_time = $12,
                enabled = $13,
                max_retries = $14,
                retry_delay = $15,
                timeout = $16,
                priority = $17,
                description = $18,
                tags = $19,
                metadata = $20
            WHERE id = $1
            RETURNING *
        """
        
        async with self.pool.acquire() as conn:
            row = await conn.fetchrow(
                sql,
                task.id,
                task.scheduler_id,
                task.task_name,
                task.task_type.value,
                task.queue_name,
                task.namespace,  # 添加namespace
                json.dumps(task.task_args),
                json.dumps(task.task_kwargs),
                task.cron_expression,
                task.interval_seconds,
                task.next_run_time,
                task.last_run_time,
                task.enabled,
                task.max_retries,
                task.retry_delay,
                task.timeout,
                task.priority,
                task.description,
                json.dumps(task.tags),
                json.dumps(task.metadata)
            )
            
            return self._row_to_task(row)
    
    async def delete_task(self, task_id: int) -> bool:
        """删除任务"""
        sql = "DELETE FROM scheduled_tasks WHERE id = $1"
        
        async with self.pool.acquire() as conn:
            result = await conn.execute(sql, task_id)
            return result.split()[-1] != '0'
    
    async def list_tasks(
        self,
        enabled: Optional[bool] = None,
        task_type: Optional[TaskType] = None,
        queue_name: Optional[str] = None,
        limit: int = 100,
        offset: int = 0
    ) -> List[ScheduledTask]:
        """列出任务"""
        conditions = []
        params = []
        param_count = 0
        
        if enabled is not None:
            param_count += 1
            conditions.append(f"enabled = ${param_count}")
            params.append(enabled)
        
        if task_type is not None:
            param_count += 1
            conditions.append(f"task_type = ${param_count}")
            params.append(task_type.value)
        
        if queue_name is not None:
            param_count += 1
            conditions.append(f"queue_name = ${param_count}")
            params.append(queue_name)
        
        where_clause = f"WHERE {' AND '.join(conditions)}" if conditions else ""
        
        param_count += 1
        limit_param = f"${param_count}"
        params.append(limit)
        
        param_count += 1
        offset_param = f"${param_count}"
        params.append(offset)
        
        sql = f"""
            SELECT * FROM scheduled_tasks
            {where_clause}
            ORDER BY created_at DESC
            LIMIT {limit_param} OFFSET {offset_param}
        """
        
        async with self.pool.acquire() as conn:
            rows = await conn.fetch(sql, *params)
            return [self._row_to_task(row) for row in rows]
    
    async def get_ready_tasks(
        self,
        batch_size: int = 100,
        lookahead_seconds: int = 60
    ) -> List[ScheduledTask]:
        """
        获取即将执行的任务
        
        Args:
            batch_size: 批次大小
            lookahead_seconds: 向前查看的秒数
        """
        cutoff_time = datetime.now() + timedelta(seconds=lookahead_seconds)
        
        sql = """
            SELECT * FROM scheduled_tasks
            WHERE enabled = true 
                AND next_run_time <= $1
                AND next_run_time IS NOT NULL
            ORDER BY next_run_time
            LIMIT $2
        """
        
        async with self.pool.acquire() as conn:
            rows = await conn.fetch(sql, cutoff_time, batch_size)
            return [self._row_to_task(row) for row in rows]
    
    async def update_task_next_run(
        self,
        task_id: int,
        next_run_time: Optional[datetime],
        last_run_time: datetime
    ):
        """更新任务的下次执行时间"""
        sql = """
            UPDATE scheduled_tasks 
            SET next_run_time = $2, last_run_time = $3
            WHERE id = $1
        """
        
        async with self.pool.acquire() as conn:
            await conn.execute(sql, task_id, next_run_time, last_run_time)
    
    async def disable_once_task(self, task_id: int):
        """禁用一次性任务（只更新必要字段）"""
        sql = """
            UPDATE scheduled_tasks 
            SET enabled = false, next_run_time = NULL
            WHERE id = $1
        """
        
        async with self.pool.acquire() as conn:
            await conn.execute(sql, task_id)
    
    async def batch_update_next_run_times(self, updates: List[tuple]):
        """批量更新任务的下次执行时间和执行次数"""
        if not updates:
            return
        
        sql = """
            UPDATE scheduled_tasks 
            SET next_run_time = u.next_run_time, 
                last_run_time = u.last_run_time,
                execution_count = COALESCE(execution_count, 0) + 1
            FROM (VALUES ($1::int, $2::timestamptz, $3::timestamptz)) AS u(id, next_run_time, last_run_time)
            WHERE scheduled_tasks.id = u.id
        """
        
        async with self.pool.acquire() as conn:
            # 使用executemany批量更新
            await conn.executemany(sql, updates)
    
    async def batch_disable_once_tasks(self, task_ids: List[int]):
        """批量禁用一次性任务"""
        if not task_ids:
            return
        
        sql = """
            UPDATE scheduled_tasks 
            SET enabled = false, next_run_time = NULL
            WHERE id = ANY($1)
        """
        
        async with self.pool.acquire() as conn:
            await conn.execute(sql, task_ids)
    
    # ==================== 执行历史操作 ====================
    
    async def record_execution(self, history: TaskExecutionHistory):
        """记录任务执行历史到tasks表"""
        # 在tasks表中创建一条新的任务记录，关联到scheduled_task
        sql = """
            INSERT INTO tasks (
                queue_name, status, scheduled_task_id, scheduled_time,
                started_at, finished_at, duration_ms, worker_id,
                error_message, retry_count, result
            ) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)
            RETURNING id
        """
        
        # 获取scheduled_task的队列名称
        task_info = await self.get_task(history.task_id)
        queue_name = task_info.queue_name if task_info else 'default'
        
        async with self.pool.acquire() as conn:
            task_id = await conn.fetchval(
                sql,
                queue_name,
                history.status.value,
                history.task_id,  # scheduled_task_id
                history.scheduled_time,
                history.started_at,
                history.finished_at,
                history.duration_ms,
                history.worker_id,
                history.error_message,
                history.retry_count,
                json.dumps(history.result) if history.result else None
            )
            return task_id
    
    async def get_task_history(
        self,
        task_id: int,
        limit: int = 100,
        status: Optional[TaskStatus] = None
    ) -> List[Dict[str, Any]]:
        """从tasks表获取任务执行历史"""
        if status:
            sql = """
                SELECT id, queue_name, status, scheduled_task_id,
                       scheduled_time, started_at, finished_at,
                       duration_ms, worker_id, error_message,
                       retry_count, result, created_at
                FROM tasks
                WHERE scheduled_task_id = $1 AND status = $2
                ORDER BY created_at DESC
                LIMIT $3
            """
            params = [task_id, status.value, limit]
        else:
            sql = """
                SELECT id, queue_name, status, scheduled_task_id,
                       scheduled_time, started_at, finished_at,
                       duration_ms, worker_id, error_message,
                       retry_count, result, created_at
                FROM tasks
                WHERE scheduled_task_id = $1
                ORDER BY created_at DESC
                LIMIT $2
            """
            params = [task_id, limit]
        
        async with self.pool.acquire() as conn:
            rows = await conn.fetch(sql, *params)
            return [dict(row) for row in rows]
    
    async def cleanup_old_history(self, days: int = 30):
        """清理旧的执行历史"""
        cutoff_date = datetime.now() - timedelta(days=days)
        
        sql = "DELETE FROM tasks WHERE scheduled_task_id IS NOT NULL AND created_at < $1"
        
        async with self.pool.acquire() as conn:
            result = await conn.execute(sql, cutoff_date)
            return int(result.split()[-1])
    
    # ==================== 批量操作方法 ====================
    
    async def batch_record_executions(self, histories: List[TaskExecutionHistory]):
        """批量记录任务执行历史到tasks表"""
        if not histories:
            return
        
        # 获取所有scheduled_task的队列和名称信息
        task_ids = list(set(h.task_id for h in histories))
        task_info_map = {}
        
        if task_ids:
            async with self.pool.acquire() as conn:
                rows = await conn.fetch(
                    "SELECT id, queue_name, task_name FROM scheduled_tasks WHERE id = ANY($1)",
                    task_ids
                )
                task_info_map = {row['id']: {'queue_name': row['queue_name'], 'task_name': row['task_name']} for row in rows}
        
        sql = """
            INSERT INTO tasks (
                id, queue_name, task_name, status, scheduled_task_id, scheduled_time,
                started_at, finished_at, duration_ms, worker_id,
                error_message, retry_count, result
            ) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)
        """
        
        # 准备批量数据
        data = []
        for history in histories:
            task_info = task_info_map.get(history.task_id, {})
            queue_name = task_info.get('queue_name', 'default')
            task_name = task_info.get('task_name', 'scheduled_task')
            
            data.append((
                history.event_id,  # id字段使用event_id
                queue_name,
                task_name,  # task_name
                history.status.value if isinstance(history.status, TaskStatus) else history.status,
                history.task_id,  # scheduled_task_id
                history.scheduled_time,
                history.started_at,
                history.finished_at,
                history.duration_ms,
                history.worker_id,
                history.error_message,
                history.retry_count,
                json.dumps(history.result) if history.result else None
            ))
        
        async with self.pool.acquire() as conn:
            # 使用executemany批量插入
            await conn.executemany(sql, data)
    
    async def batch_update_tasks(self, tasks: List[ScheduledTask]):
        """批量更新任务"""
        if not tasks:
            return
        
        sql = """
            UPDATE scheduled_tasks SET
                scheduler_id = $2,
                task_name = $3,
                task_type = $4,
                queue_name = $5,
                task_args = $6,
                task_kwargs = $7,
                cron_expression = $8,
                interval_seconds = $9,
                next_run_time = $10,
                last_run_time = $11,
                enabled = $12,
                max_retries = $13,
                retry_delay = $14,
                timeout = $15,
                description = $16,
                metadata = $17,
                updated_at = $18
            WHERE id = $1
        """
        
        # 准备批量数据
        data = []
        now = datetime.now()
        for task in tasks:
            data.append((
                task.id,
                task.scheduler_id,
                task.task_name,
                task.task_type.value if isinstance(task.task_type, TaskType) else task.task_type,
                task.queue_name,
                json.dumps(task.task_args) if task.task_args else '[]',
                json.dumps(task.task_kwargs) if task.task_kwargs else '{}',
                task.cron_expression,
                task.interval_seconds,
                task.next_run_time,
                task.last_run_time,
                task.enabled,
                task.max_retries,
                task.retry_delay,
                task.timeout,
                task.description,
                json.dumps(task.metadata) if task.metadata else None,
                now
            ))
        
        async with self.pool.acquire() as conn:
            # 使用executemany批量更新
            await conn.executemany(sql, data)
    
    # ==================== 辅助方法 ====================
    
    def _row_to_task(self, row) -> ScheduledTask:
        """将数据库行转换为ScheduledTask对象"""
        from decimal import Decimal
        
        # 处理interval_seconds的Decimal类型
        interval_seconds = row['interval_seconds']
        if interval_seconds is not None and isinstance(interval_seconds, Decimal):
            interval_seconds = float(interval_seconds)
        
        return ScheduledTask(
            id=row['id'],
            scheduler_id=row['scheduler_id'],
            task_name=row['task_name'],
            task_type=TaskType(row['task_type']),
            queue_name=row['queue_name'],
            namespace=row.get('namespace', 'default'),  # 添加namespace字段
            task_args=row['task_args'] if isinstance(row['task_args'], list) else json.loads(row['task_args']),
            task_kwargs=row['task_kwargs'] if isinstance(row['task_kwargs'], dict) else json.loads(row['task_kwargs']),
            cron_expression=row['cron_expression'],
            interval_seconds=interval_seconds,
            next_run_time=row['next_run_time'],
            last_run_time=row['last_run_time'],
            enabled=row['enabled'],
            max_retries=row['max_retries'],
            retry_delay=row['retry_delay'],
            timeout=row['timeout'],
            priority=row.get('priority'),
            description=row['description'],
            tags=row['tags'] if isinstance(row['tags'], list) else (json.loads(row['tags']) if row['tags'] else []),
            metadata=row['metadata'] if isinstance(row['metadata'], dict) else (json.loads(row['metadata']) if row['metadata'] else None),
            created_at=row['created_at'],
            updated_at=row['updated_at']
        )
    
    def _row_to_history(self, row) -> Dict[str, Any]:
        """将数据库行转换为历史记录字典"""
        return {
            'id': row['id'],
            'scheduled_task_id': row['scheduled_task_id'],
            'status': row['status'],
            'scheduled_time': row['scheduled_time'],
            'started_at': row['started_at'],
            'finished_at': row['finished_at'],
            'duration_ms': row['duration_ms'],
            'worker_id': row['worker_id'],
            'error_message': row['error_message'],
            'retry_count': row['retry_count'],
            'result': row['result'] if isinstance(row['result'], dict) else json.loads(row['result']) if row['result'] else None,
            'created_at': row['created_at']
        }
    
    async def create_or_get_task(self, task: ScheduledTask, skip_if_exists: bool = True) -> tuple[ScheduledTask, bool]:
        """
        创建任务或获取已存在的任务
        
        Args:
            task: 任务对象
            skip_if_exists: 如果任务已存在是否跳过（True=跳过，False=抛出异常）
            
        Returns:
            (task, created): 任务对象和是否新创建的标志
        """
        if task.scheduler_id:
            # 先检查是否已存在
            existing = await self.get_task_by_scheduler_id(task.scheduler_id)
            if existing:
                if skip_if_exists:
                    return existing, False
                else:
                    raise ValueError(f"Task with scheduler_id '{task.scheduler_id}' already exists")
        
        # 创建新任务
        created_task = await self.create_task(task)
        return created_task, True
    
    async def batch_create_tasks(self, tasks: List[ScheduledTask], skip_existing: bool = True) -> List[ScheduledTask]:
        """
        批量创建任务（优化版本）
        
        Args:
            tasks: 任务列表
            skip_existing: 是否跳过已存在的任务
            
        Returns:
            成功创建的任务列表
        """
        if not tasks:
            return []
        
        async with self.pool.acquire() as conn:
            # 1. 批量查询已存在的scheduler_id
            scheduler_ids = [t.scheduler_id for t in tasks if t.scheduler_id]
            existing_ids = set()
            
            if scheduler_ids and skip_existing:
                rows = await conn.fetch(
                    "SELECT scheduler_id FROM scheduled_tasks WHERE scheduler_id = ANY($1)",
                    scheduler_ids
                )
                existing_ids = {row['scheduler_id'] for row in rows}
                
            # 2. 过滤出需要创建的任务
            tasks_to_create = []
            for task in tasks:
                if task.scheduler_id in existing_ids:
                    continue  # 跳过已存在的
                tasks_to_create.append(task)
            
            if not tasks_to_create:
                return []
            
            # 3. 准备批量插入的数据
            values = []
            for task in tasks_to_create:
                values.append((
                    task.scheduler_id,
                    task.task_name,
                    task.task_type.value,
                    task.queue_name,
                    json.dumps(task.task_args),
                    json.dumps(task.task_kwargs),
                    task.cron_expression,
                    task.interval_seconds,
                    task.next_run_time,
                    task.enabled,
                    task.max_retries,
                    task.retry_delay,
                    task.timeout,
                    task.priority,
                    task.description,
                    json.dumps(task.tags),
                    json.dumps(task.metadata)
                ))
            
            # 4. 批量插入（使用UNNEST进行批量插入）
            created_rows = await conn.fetch(
                """
                INSERT INTO scheduled_tasks (
                    scheduler_id, task_name, task_type, queue_name,
                    task_args, task_kwargs, cron_expression, interval_seconds,
                    next_run_time, enabled, max_retries, retry_delay, timeout,
                    priority, description, tags, metadata
                )
                SELECT * FROM UNNEST(
                    $1::text[], $2::text[], $3::text[], $4::text[],
                    $5::jsonb[], $6::jsonb[], $7::text[], $8::numeric[],
                    $9::timestamptz[], $10::boolean[], $11::int[], $12::int[], $13::int[],
                    $14::int[], $15::text[], $16::jsonb[], $17::jsonb[]
                ) AS t(
                    scheduler_id, task_name, task_type, queue_name,
                    task_args, task_kwargs, cron_expression, interval_seconds,
                    next_run_time, enabled, max_retries, retry_delay, timeout,
                    priority, description, tags, metadata
                )
                ON CONFLICT (scheduler_id) DO NOTHING
                RETURNING *
                """,
                # 解包values为列数组
                [v[0] for v in values],  # scheduler_id
                [v[1] for v in values],  # task_name
                [v[2] for v in values],  # task_type
                [v[3] for v in values],  # queue_name
                [v[4] for v in values],  # task_args
                [v[5] for v in values],  # task_kwargs
                [v[6] for v in values],  # cron_expression
                [v[7] for v in values],  # interval_seconds
                [v[8] for v in values],  # next_run_time
                [v[9] for v in values],  # enabled
                [v[10] for v in values],  # max_retries
                [v[11] for v in values],  # retry_delay
                [v[12] for v in values],  # timeout
                [v[13] for v in values],  # priority
                [v[14] for v in values],  # description
                [v[15] for v in values],  # tags
                [v[16] for v in values],  # metadata
            )
            
            # 5. 转换结果为任务对象
            created_tasks = [self._row_to_task(row) for row in created_rows]
            
            return created_tasks


"""
多命名空间调度器管理器
自动检测和管理多个命名空间的调度器实例
"""
import asyncio
import logging
import multiprocessing
import traceback
from typing import Dict, Set

from jettask import Jettask
from jettask.task_center import TaskCenter
from jettask.scheduler.scheduler import TaskScheduler
from jettask.scheduler.manager import ScheduledTaskManager

logger = logging.getLogger(__name__)


class MultiNamespaceSchedulerManager:
    """多命名空间调度器管理器"""
    
    def __init__(self, 
                 task_center_base_url: str,
                 check_interval: int = 30,
                 scheduler_interval: float = 0.1,
                 batch_size: int = 100,
                 debug: bool = False):
        """
        初始化多命名空间调度器管理器
        
        Args:
            task_center_base_url: 任务中心基础URL (如 http://localhost:8001)
            check_interval: 命名空间检测间隔（秒）
            scheduler_interval: 调度器扫描间隔（秒）
            batch_size: 每批处理的最大任务数
            debug: 是否启用调试模式
        """
        self.task_center_base_url = task_center_base_url.rstrip('/')
        self.check_interval = check_interval
        self.scheduler_interval = scheduler_interval
        self.batch_size = batch_size
        self.debug = debug
        
        # 存储每个命名空间的进程
        self.scheduler_processes: Dict[str, multiprocessing.Process] = {}
        self.running = False
        
        # 设置日志级别
        if debug:
            logging.basicConfig(level=logging.DEBUG)
        else:
            logging.basicConfig(level=logging.INFO)
    
    async def get_active_namespaces(self) -> Set[str]:
        """获取所有活跃的命名空间"""
        import aiohttp
        
        try:
            # 直接调用API获取命名空间列表
            url = f"{self.task_center_base_url}/api/namespaces"
            
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as response:
                    if response.status == 200:
                        namespaces = await response.json()
                        if namespaces:
                            active_names = {ns['name'] for ns in namespaces if ns.get('name')}
                            logger.info(f"发现 {len(active_names)} 个活跃的命名空间: {active_names}")
                            return active_names
                        else:
                            logger.warning("没有找到任何命名空间")
                            return set()
                    else:
                        logger.error(f"获取命名空间列表失败，状态码: {response.status}")
                        return set()
                        
        except Exception as e:
            logger.error(f"获取命名空间列表失败: {e}")
            return set()
    
    def start_scheduler_for_namespace(self, namespace: str):
        """为指定命名空间启动调度器进程"""
        if namespace in self.scheduler_processes:
            # 检查进程是否还在运行
            if self.scheduler_processes[namespace].is_alive():
                return
            else:
                # 清理已停止的进程
                logger.info(f"清理已停止的调度器进程: {namespace}")
                self.scheduler_processes[namespace].terminate()
                self.scheduler_processes[namespace].join(timeout=5)
                del self.scheduler_processes[namespace]
        
        # 创建新进程
        process = multiprocessing.Process(
            target=run_scheduler_for_namespace,
            args=(
                namespace,
                self.task_center_base_url,
                self.scheduler_interval,
                self.batch_size,
                self.debug
            ),
            name=f"scheduler_{namespace}"
        )
        
        process.start()
        self.scheduler_processes[namespace] = process
        logger.info(f"启动命名空间 {namespace} 的调度器进程, PID: {process.pid}")
    
    def stop_scheduler_for_namespace(self, namespace: str):
        """停止指定命名空间的调度器进程"""
        if namespace in self.scheduler_processes:
            process = self.scheduler_processes[namespace]
            if process.is_alive():
                logger.info(f"停止命名空间 {namespace} 的调度器进程")
                process.terminate()
                process.join(timeout=10)
                
                if process.is_alive():
                    logger.warning(f"强制停止命名空间 {namespace} 的调度器进程")
                    process.kill()
                    process.join(timeout=5)
            
            del self.scheduler_processes[namespace]
    
    async def check_and_update_schedulers(self):
        """检查并更新调度器（添加新的，停止已删除的）"""
        # 获取当前活跃的命名空间
        active_namespaces = await self.get_active_namespaces()
        current_namespaces = set(self.scheduler_processes.keys())
        
        # 找出需要添加的命名空间
        to_add = active_namespaces - current_namespaces
        # 找出需要删除的命名空间
        to_remove = current_namespaces - active_namespaces
        
        # 启动新的调度器
        for namespace in to_add:
            logger.info(f"检测到新命名空间: {namespace}")
            self.start_scheduler_for_namespace(namespace)
        
        # 停止已删除的调度器
        for namespace in to_remove:
            logger.info(f"检测到命名空间已删除: {namespace}")
            self.stop_scheduler_for_namespace(namespace)
        
        # 检查现有进程的健康状态
        for namespace in active_namespaces & current_namespaces:
            process = self.scheduler_processes.get(namespace)
            if process and not process.is_alive():
                logger.warning(f"调度器进程 {namespace} 已停止，重新启动")
                del self.scheduler_processes[namespace]
                self.start_scheduler_for_namespace(namespace)
    
    async def run(self):
        """运行多命名空间调度器管理器"""
        self.running = True
        logger.info(f"启动多命名空间调度器管理器")
        logger.info(f"任务中心: {self.task_center_base_url}")
        logger.info(f"命名空间检测间隔: {self.check_interval} 秒")
        logger.info(f"调度器扫描间隔: {self.scheduler_interval} 秒")
        logger.info(f"批处理大小: {self.batch_size}")
        
        # 初始检查和启动
        await self.check_and_update_schedulers()
        
        # 定期检查命名空间变化
        while self.running:
            try:
                await asyncio.sleep(self.check_interval)
                await self.check_and_update_schedulers()
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"检查命名空间时出错: {e}")
                if self.debug:
                    traceback.print_exc()
    
    def stop(self):
        """停止管理器和所有调度器"""
        logger.info("停止多命名空间调度器管理器")
        self.running = False
        
        # 停止所有调度器进程
        for namespace in list(self.scheduler_processes.keys()):
            self.stop_scheduler_for_namespace(namespace)
        
        logger.info("所有调度器已停止")


def run_scheduler_for_namespace(namespace: str, 
                               task_center_base_url: str,
                               interval: float,
                               batch_size: int,
                               debug: bool):
    """在独立进程中运行指定命名空间的调度器"""
    import asyncio
    import logging
    import signal
    import sys
    
    # 设置进程标题（如果可用）
    try:
        import setproctitle  # type: ignore
        setproctitle.setproctitle(f"jettask-scheduler-{namespace}")
    except ImportError:
        pass
    
    # 配置日志
    logging.basicConfig(
        level=logging.DEBUG if debug else logging.INFO,
        format=f'%(asctime)s - %(levelname)s - [{namespace}] %(message)s'
    )
    logger = logging.getLogger(__name__)
    
    async def run_scheduler():
        """运行调度器的异步函数"""
        scheduler_instance = None
        try:
            # 构建命名空间特定的URL
            task_center_url = f"{task_center_base_url}/api/v1/namespaces/{namespace}"
            logger.info(f"连接到任务中心: {task_center_url}")
            
            # 连接任务中心
            tc = TaskCenter(task_center_url)
            if not tc._connect_sync():
                logger.error(f"无法连接到任务中心: {namespace}")
                return
            
            logger.info(f"成功连接到命名空间: {tc.namespace_name}")
            
            # 创建app实例
            app = Jettask(task_center=tc)
            
            if not app.redis_url or not app.pg_url:
                logger.error(f"任务中心配置不完整: {namespace}")
                return
            
            # 显示配置信息
            logger.info(f"命名空间 {namespace} 的调度器配置:")
            logger.info(f"  Redis: {app.redis_url}")
            logger.info(f"  PostgreSQL: {app.pg_url}")
            logger.info(f"  间隔: {interval} 秒")
            logger.info(f"  批大小: {batch_size}")
            
            # 创建调度器实例
            manager = ScheduledTaskManager(app)
            scheduler_instance = TaskScheduler(
                app=app,
                db_manager=manager,
                scan_interval=interval,
                batch_size=batch_size
            )
            
            # 运行调度器（run方法内部会处理连接）
            logger.info(f"启动命名空间 {namespace} 的调度器...")
            await scheduler_instance.run()
            
        except asyncio.CancelledError:
            logger.info(f"调度器 {namespace} 收到取消信号")
        except KeyboardInterrupt:
            logger.info(f"调度器 {namespace} 收到中断信号")
        except Exception as e:
            logger.error(f"调度器 {namespace} 运行错误: {e}")
            if debug:
                traceback.print_exc()
        finally:
            # 清理资源
            if scheduler_instance:
                scheduler_instance.stop()
            
            logger.info(f"调度器 {namespace} 已停止")
    
    # 设置信号处理
    import signal
    import sys
    
    def signal_handler(signum, frame):
        logger.info(f"调度器 {namespace} 收到信号 {signum}")
        sys.exit(0)
    
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    
    # 运行调度器
    try:
        asyncio.run(run_scheduler())
    except (KeyboardInterrupt, SystemExit):
        logger.info(f"调度器 {namespace} 正常退出")
    except Exception as e:
        logger.error(f"调度器 {namespace} 异常退出: {e}")
        if debug:
            traceback.print_exc()


"""
Namespace management API v1
"""
from typing import List, Optional
from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.ext.asyncio import AsyncSession

from dependencies import (
    get_database_manager, get_request_metrics, RequestMetrics,
    validate_page_params
)
from models.requests import NamespaceCreateRequest, NamespaceUpdateRequest, NamespaceListRequest
from models.responses import NamespaceListResponse, NamespaceResponse, BaseResponse
from core.cache import cache_result, invalidate_cache, CACHE_CONFIGS
from core.exceptions import NamespaceNotFoundError, ValidationError
from namespace_data_access import get_namespace_data_access
import logging

logger = logging.getLogger(__name__)
router = APIRouter()


@router.get("", response_model=NamespaceListResponse)
@cache_result(**CACHE_CONFIGS['namespace_config'])
async def list_namespaces(
    page: int = Query(1, ge=1, description="页码"),
    page_size: int = Query(20, ge=1, le=100, description="每页大小"),
    is_active: Optional[bool] = Query(None, description="是否启用"),
    search: Optional[str] = Query(None, description="搜索关键词"),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取命名空间列表"""
    metrics.start("system", "GET /namespaces")
    
    try:
        namespace_data_access = get_namespace_data_access()
        
        # 获取所有命名空间配置
        all_namespaces = await namespace_data_access.list_namespaces()
        
        # 转换为响应格式
        namespace_list = []
        for ns_config in all_namespaces:
            namespace_info = {
                'id': ns_config.get('id', ns_config['name']),
                'name': ns_config['name'],
                'display_name': ns_config.get('display_name', ns_config['name']),
                'description': ns_config.get('description'),
                'redis_url': ns_config['redis_url'],
                'pg_url': ns_config.get('pg_url'),
                'created_at': ns_config.get('created_at'),
                'is_active': ns_config.get('is_active', True),
                'queue_count': 0,  # 可以后续查询实际数量
                'task_count': 0   # 可以后续查询实际数量
            }
            
            # 应用搜索筛选
            if search:
                if (search.lower() not in namespace_info['name'].lower() and 
                    search.lower() not in (namespace_info['display_name'] or '').lower()):
                    continue
            
            # 应用状态筛选
            if is_active is not None and namespace_info['is_active'] != is_active:
                continue
            
            namespace_list.append(namespace_info)
        
        # 分页
        total = len(namespace_list)
        start = (page - 1) * page_size
        end = start + page_size
        paginated_namespaces = namespace_list[start:end]
        
        return NamespaceListResponse.create(
            data=paginated_namespaces,
            total=total,
            page=page,
            page_size=page_size
        )
        
    except Exception as e:
        logger.error(f"获取命名空间列表失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.get("/{namespace_name}", response_model=NamespaceResponse)
@cache_result(**CACHE_CONFIGS['namespace_config'])
async def get_namespace_detail(
    namespace_name: str,
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """获取命名空间详情"""
    metrics.start("system", f"GET /namespaces/{namespace_name}")
    
    try:
        namespace_data_access = get_namespace_data_access()
        
        # 获取命名空间配置
        ns_config = await namespace_data_access.get_namespace_config(namespace_name)
        if not ns_config:
            raise NamespaceNotFoundError(namespace_name)
        
        # 获取连接以检查状态和统计信息
        try:
            conn = await namespace_data_access.manager.get_connection(namespace_name)
            
            # 获取队列统计
            queue_count = 0
            task_count = 0
            
            try:
                # 从Redis获取队列数量
                redis_client = await conn.get_redis_client(decode=False)
                try:
                    # 查询所有队列键
                    pattern = f"{conn.redis_prefix}:QUEUE:*"
                    keys = await redis_client.keys(pattern)
                    
                    # 去重基础队列名
                    base_queues = set()
                    for key in keys:
                        key_str = key.decode('utf-8') if isinstance(key, bytes) else key
                        parts = key_str.split(':')
                        if len(parts) >= 3:
                            queue_part = ':'.join(parts[2:])
                            # 去除优先级后缀
                            if ':' in queue_part:
                                base_part = queue_part.rsplit(':', 1)
                                if base_part[1].isdigit():
                                    base_queues.add(base_part[0])
                                else:
                                    base_queues.add(queue_part)
                            else:
                                base_queues.add(queue_part)
                    
                    queue_count = len(base_queues)
                finally:
                    await redis_client.aclose()
                
                # 从PostgreSQL获取任务数量
                if conn.AsyncSessionLocal:
                    async with conn.AsyncSessionLocal() as session:
                        result = await session.execute("SELECT COUNT(*) FROM tasks")
                        row = result.fetchone()
                        if row:
                            task_count = row[0]
            
            except Exception as e:
                logger.warning(f"获取命名空间 {namespace_name} 统计信息失败: {e}")
        
        except Exception as e:
            logger.warning(f"连接命名空间 {namespace_name} 失败: {e}")
        
        # 构造响应
        namespace_detail = {
            'id': ns_config.get('id', namespace_name),
            'name': namespace_name,
            'display_name': ns_config.get('display_name', namespace_name),
            'description': ns_config.get('description'),
            'redis_url': ns_config['redis_url'],
            'pg_url': ns_config.get('pg_url'),
            'created_at': ns_config.get('created_at'),
            'is_active': ns_config.get('is_active', True),
            'queue_count': queue_count,
            'task_count': task_count
        }
        
        return NamespaceResponse(data=namespace_detail)
        
    except NamespaceNotFoundError:
        raise
    except Exception as e:
        logger.error(f"获取命名空间详情失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.post("", response_model=NamespaceResponse)
@invalidate_cache("namespace_config")
async def create_namespace(
    request: NamespaceCreateRequest,
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """创建命名空间"""
    metrics.start("system", "POST /namespaces")
    
    try:
        namespace_data_access = get_namespace_data_access()
        
        # 检查命名空间是否已存在
        existing_config = await namespace_data_access.get_namespace_config(request.name)
        if existing_config:
            raise ValidationError(f"Namespace '{request.name}' already exists")
        
        # 验证连接配置
        await _validate_connection_config(request.redis_url, request.pg_url)
        
        # 创建命名空间配置
        ns_config = {
            'name': request.name,
            'display_name': request.display_name,
            'description': request.description,
            'redis_url': request.redis_url,
            'pg_url': request.pg_url,
            'redis_prefix': request.redis_prefix,
            'is_active': True
        }
        
        created_config = await namespace_data_access.create_namespace(ns_config)
        
        # 构造响应
        namespace_detail = {
            'id': created_config.get('id', request.name),
            'name': request.name,
            'display_name': request.display_name,
            'description': request.description,
            'redis_url': request.redis_url,
            'pg_url': request.pg_url,
            'created_at': created_config.get('created_at'),
            'is_active': True,
            'queue_count': 0,
            'task_count': 0
        }
        
        return NamespaceResponse(
            data=namespace_detail,
            message="Namespace created successfully"
        )
        
    except ValidationError:
        raise
    except Exception as e:
        logger.error(f"创建命名空间失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.put("/{namespace_name}", response_model=NamespaceResponse)
@invalidate_cache("namespace_config")
async def update_namespace(
    namespace_name: str,
    request: NamespaceUpdateRequest,
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """更新命名空间"""
    metrics.start("system", f"PUT /namespaces/{namespace_name}")
    
    try:
        namespace_data_access = get_namespace_data_access()
        
        # 检查命名空间是否存在
        existing_config = await namespace_data_access.get_namespace_config(namespace_name)
        if not existing_config:
            raise NamespaceNotFoundError(namespace_name)
        
        # 构造更新数据
        update_data = {}
        if request.display_name is not None:
            update_data['display_name'] = request.display_name
        if request.description is not None:
            update_data['description'] = request.description
        if request.is_active is not None:
            update_data['is_active'] = request.is_active
        
        # 验证连接配置更新
        if request.redis_url or request.pg_url:
            redis_url = request.redis_url or existing_config['redis_url']
            pg_url = request.pg_url or existing_config.get('pg_url')
            await _validate_connection_config(redis_url, pg_url)
            
            if request.redis_url:
                update_data['redis_url'] = request.redis_url
            if request.pg_url:
                update_data['pg_url'] = request.pg_url
        
        # 执行更新
        updated_config = await namespace_data_access.update_namespace(namespace_name, update_data)
        
        # 构造响应
        namespace_detail = {
            'id': updated_config.get('id', namespace_name),
            'name': namespace_name,
            'display_name': updated_config.get('display_name', namespace_name),
            'description': updated_config.get('description'),
            'redis_url': updated_config['redis_url'],
            'pg_url': updated_config.get('pg_url'),
            'created_at': updated_config.get('created_at'),
            'is_active': updated_config.get('is_active', True),
            'queue_count': 0,  # 可以后续查询实际数量
            'task_count': 0   # 可以后续查询实际数量
        }
        
        return NamespaceResponse(
            data=namespace_detail,
            message="Namespace updated successfully"
        )
        
    except NamespaceNotFoundError:
        raise
    except ValidationError:
        raise
    except Exception as e:
        logger.error(f"更新命名空间失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.delete("/{namespace_name}", response_model=BaseResponse)
@invalidate_cache("namespace_config")
async def delete_namespace(
    namespace_name: str,
    force: bool = Query(False, description="是否强制删除"),
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """删除命名空间"""
    metrics.start("system", f"DELETE /namespaces/{namespace_name}")
    
    try:
        if namespace_name == "default":
            raise ValidationError("Cannot delete the default namespace")
        
        namespace_data_access = get_namespace_data_access()
        
        # 检查命名空间是否存在
        existing_config = await namespace_data_access.get_namespace_config(namespace_name)
        if not existing_config:
            raise NamespaceNotFoundError(namespace_name)
        
        # 检查是否有活跃的队列或任务（如果不是强制删除）
        if not force:
            try:
                conn = await namespace_data_access.manager.get_connection(namespace_name)
                
                # 检查Redis中是否有队列
                redis_client = await conn.get_redis_client(decode=False)
                try:
                    pattern = f"{conn.redis_prefix}:QUEUE:*"
                    keys = await redis_client.keys(pattern)
                    if keys:
                        raise ValidationError(
                            f"Namespace has {len(keys)} active queues. Use force=true to delete anyway."
                        )
                finally:
                    await redis_client.aclose()
                
                # 检查PostgreSQL中是否有任务
                if conn.AsyncSessionLocal:
                    async with conn.AsyncSessionLocal() as session:
                        result = await session.execute("SELECT COUNT(*) FROM tasks")
                        row = result.fetchone()
                        if row and row[0] > 0:
                            raise ValidationError(
                                f"Namespace has {row[0]} tasks. Use force=true to delete anyway."
                            )
            
            except ValidationError:
                raise
            except Exception as e:
                logger.warning(f"检查命名空间 {namespace_name} 资源失败: {e}")
        
        # 执行删除
        success = await namespace_data_access.delete_namespace(namespace_name)
        if not success:
            raise HTTPException(status_code=500, detail="Failed to delete namespace")
        
        return BaseResponse(
            message=f"Namespace '{namespace_name}' deleted successfully"
        )
        
    except NamespaceNotFoundError:
        raise
    except ValidationError:
        raise
    except Exception as e:
        logger.error(f"删除命名空间失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


@router.post("/{namespace_name}/test-connection", response_model=BaseResponse)
async def test_namespace_connection(
    namespace_name: str,
    metrics: RequestMetrics = Depends(get_request_metrics)
):
    """测试命名空间连接"""
    metrics.start("system", f"POST /namespaces/{namespace_name}/test-connection")
    
    try:
        namespace_data_access = get_namespace_data_access()
        
        # 获取命名空间配置
        ns_config = await namespace_data_access.get_namespace_config(namespace_name)
        if not ns_config:
            raise NamespaceNotFoundError(namespace_name)
        
        # 测试连接
        connection_results = await _test_connections(
            ns_config['redis_url'],
            ns_config.get('pg_url')
        )
        
        overall_status = "healthy" if all(r['status'] == 'healthy' for r in connection_results.values()) else "unhealthy"
        
        return BaseResponse(
            data={
                'namespace': namespace_name,
                'overall_status': overall_status,
                'connections': connection_results
            },
            message=f"Connection test completed for namespace '{namespace_name}'"
        )
        
    except NamespaceNotFoundError:
        raise
    except Exception as e:
        logger.error(f"测试命名空间连接失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))
    finally:
        metrics.finish()


# 辅助函数

async def _validate_connection_config(redis_url: str, pg_url: Optional[str] = None):
    """验证连接配置"""
    connection_results = await _test_connections(redis_url, pg_url)
    
    failed_connections = [
        name for name, result in connection_results.items()
        if result['status'] != 'healthy'
    ]
    
    if failed_connections:
        raise ValidationError(
            f"Connection validation failed for: {', '.join(failed_connections)}"
        )


async def _test_connections(redis_url: str, pg_url: Optional[str] = None) -> dict:
    """测试数据库连接"""
    results = {}
    
    # 测试Redis连接
    try:
        import redis.asyncio as redis
        redis_client = redis.from_url(redis_url, decode_responses=False)
        await redis_client.ping()
        await redis_client.aclose()
        results['redis'] = {
            'status': 'healthy',
            'url': redis_url,
            'message': 'Connection successful'
        }
    except Exception as e:
        results['redis'] = {
            'status': 'unhealthy',
            'url': redis_url,
            'message': f'Connection failed: {str(e)}'
        }
    
    # 测试PostgreSQL连接（如果提供）
    if pg_url:
        try:
            import asyncpg
            
            # 解析连接字符串
            if pg_url.startswith('postgresql://'):
                connection_string = pg_url.replace('postgresql://', '')
            elif pg_url.startswith('postgresql+asyncpg://'):
                connection_string = pg_url.replace('postgresql+asyncpg://', '')
            else:
                connection_string = pg_url
            
            auth, host_info = connection_string.split('@')
            user, password = auth.split(':')
            host_port, database = host_info.split('/')
            host, port = host_port.split(':')
            
            conn = await asyncpg.connect(
                host=host, port=int(port), user=user, password=password, database=database
            )
            await conn.execute("SELECT 1")
            await conn.close()
            
            results['postgresql'] = {
                'status': 'healthy',
                'url': pg_url,
                'message': 'Connection successful'
            }
        except Exception as e:
            results['postgresql'] = {
                'status': 'unhealthy', 
                'url': pg_url,
                'message': f'Connection failed: {str(e)}'
            }
    
    return results